import bb.cascades 1.2
import bb.system 1.2
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0

Page {
    property bool selfPosts
    property bool allowAtEndSignal
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    objectName: "accountPage"
    id: accountPage
    actions: [
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToTop
                }
            ]
            title: "To Top"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_top.png"
            onTriggered: {
                accountList.scrollToItem([ 0 ]);
            }
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToBottom
                }
            ]
            title: "To Bottom"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_bottom.png"
            onTriggered: {
                accountList.scrollToItem([ accountList.dataModel.size() - 1 ]);
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("r")
                }
            ]
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: "Refresh"
            imageSource: "asset:///images/reloadIcon.png"
            onTriggered: {
                feedHttp.getUserFeed(accountBar.selectedTab, accountActivity, accountList);
            }
        }
    ]
    Container {
        layout: StackLayout {
        }
        Container {
            bottomPadding: 3 //2 was specified for the 720x720 version of this file
            id: highlightContainer
            background: Color.create("#0C87BF")
            //preferredWidth: 768
            horizontalAlignment: HorizontalAlignment.Fill
            Container {
                objectName: "titlebarContainer"
                id: titlebarContainer
                property alias usernameP: accountBar.username
                property alias commentKarmaP: accountBar.comKarma
                property alias linkKarmaP: accountBar.linkKarma
                AccountBarView {
                    id: accountBar
                    onSelectedTabChanged: {
                        if (selectedTab == "submitted") {
                            selfPosts = true;
                        } else {
                            selfPosts = false;
                        }
                        feedHttp.getUserFeed(selectedTab, accountActivity, accountList);
                    }
                    onScrolling: {
                        if (scrolling) {
                            Qt.tabbedPane.peekEnabled = false;
                            //navigationPaneTab2.peekEnabled = false;
                        } else {
                            Qt.tabbedPane.peekEnabled = true;
                        }
                    }
                    onUsernameChanged: {
                        feedHttp.getUserFeed("comments", accountActivity, accountList);
                    }
                }
            }
        }
        Container {
            layout: DockLayout {
            }
            ListView {
                property variant fontSize: getFontsize()
                scrollRole: ScrollRole.Main
                id: accountList
                listItemComponents: [
                    ListItemComponent {
                        id: listItemRoot
                        type: "thread"
                        FeedListItem {
                            id: feedListItem
                            fontsize: feedListItem.ListItem.view.fontSize
                            showSubreddit: true
                            contextActions: [
                                ActionSet {
                                    title: ListItemData.data.title
                                    subtitle: ListItemData.data.author
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("u")
                                            }
                                        ]
                                        id: upvoteAction
                                        title: "Upvote"
                                        imageSource: "asset:///images/upvoteIcon.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToast.body = "You must login to vote!";
                                                accountLoginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == true) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == false) {
                                                    postActions.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.score += 2;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActions.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.score += 1;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("d")
                                            }
                                        ]
                                        id: downvoteAction
                                        title: "Downvote"
                                        imageSource: "asset:///images/downvoteIcon.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToast.body = "You must login to vote!";
                                                accountLoginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == false) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == true) {
                                                    postActions.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.score -= 2;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActions.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.score -= 1;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("c")
                                            }
                                        ]
                                        title: "Comments (" + ListItemData.data.num_comments + ")"
                                        imageSource: "asset:///images/commentsIcon.png"
                                        onTriggered: {
                                            var selectedItem = ListItemData;
                                            var page = listFeedDetailsViewList.createObject();
                                            page.pageTitle = selectedItem.data.title;
                                            page.postID = (selectedItem.data.name);
                                            feedListItem.ListItem.view.pushPageStack(page);
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("s")
                                            }
                                        ]
                                        id: saveAction
                                        title: "Save"
                                        imageSource: "asset:///images/star.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToast.body = "You must login to save posts!";
                                                accountLoginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.saved == false) {
                                                    postActions.savePost(selectedItem.data.name, "save");
                                                    selectedItem.data.saved = true;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActions.savePost(selectedItem.data.name, "unsave");
                                                    selectedItem.data.saved = false;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("h")
                                            }
                                        ]
                                        id: hideAction
                                        title: "Hide"
                                        imageSource: "asset:///images/hide.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToast.body = "You must login to hide posts!";
                                                accountLoginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.hidden == false) {
                                                    postActions.hidePost(true, selectedItem.data.name);
                                                    feedListItem.ListItem.view.dataModel.removeAt(feedListItem.ListItem.indexPath);
                                                } else {
                                                    postActions.hidePost(false, selectedItem.data.name);
                                                    feedListItem.ListItem.view.dataModel.removeAt(feedListItem.ListItem.indexPath);
                                                }
                                            }
                                        }
                                    }
                                    InvokeActionItem {
                                        id: shareAction
                                        objectName: "shareAction"
                                        title: "Share"
                                        query {
                                            mimeType: "text/plain"
                                            invokeActionId: "bb.action.SHARE"
                                        }
                                        onTriggered: {
                                            data = ListItemData.data.title + " - https://www.reddit.com" + ListItemData.data.permalink + " via Reddit in Motion";
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("g")
                                            }
                                        ]
                                        id: gotoAction
                                        imageSource: "asset:///images/gotoIcon.png"
                                        objectName: "gotoAction"
                                        title: "Goto /r/" + ListItemData.data.subreddit
                                        //enabled: feedListShowSub
                                        onTriggered: {
                                            var selectedItem = ListItemData;
                                            var page = feedViewGoto.createObject();
                                            var subUrl = selectedItem.data.subreddit;
                                            var firstSlash = subUrl.lastIndexOf("/");
                                            if (firstSlash == -1) {
                                                page.subUrl = "/r/" + subUrl + "/";
                                            } else {
                                                page.subUrl = "/r" + subUrl.substring(firstSlash) + "/";
                                            }
                                            //page.appBackup = app;
                                            feedListItem.ListItem.view.pushPageStack(page);
                                        }
                                    }
                                    attachedObjects: [
                                        SystemToast {
                                            id: accountLoginToast
                                            body: "You must login to vote!"
                                        },
                                        ComponentDefinition {
                                            id: feedViewGoto
                                            source: "FeedView.qml"
                                        }
                                    ]
                                }
                            ]
                            attachedObjects: [
                                ComponentDefinition {
                                    id: listFeedDetailsViewList
                                    source: "FeedDetails.qml"
                                },
                                PostActions {
                                    id: postActions
                                }
                            ]
                            contextMenuHandler: ContextMenuHandler {
                                id: accountContextHandler
                                onPopulating: {
                                    if (ListItemData.data.likes == true) {
                                        upvoteAction.setImageSource("asset:///images/upvoteSelectedIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    } else if (ListItemData.data.likes == false) {
                                        downvoteAction.setImageSource("asset:///images/downvoteSelectedIcon.png");
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                    } else {
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    }
                                    if (ListItemData.data.saved == true) {
                                        saveAction.setImageSource("asset:///images/saved.png");
                                        saveAction.setTitle("Unsave");
                                    } else {
                                        saveAction.setImageSource("asset:///images/star.png");
                                        saveAction.setTitle("Save");
                                    }
                                    if (ListItemData.data.hidden == true) {
                                        hideAction.setImageSource("asset:///images/unhide.png");
                                        hideAction.setTitle("Unhide");
                                    } else {
                                        hideAction.setImageSource("asset:///images/hide.png");
                                        hideAction.setTitle("Hide");
                                    }
                                }
                            }
                            onCommentBubbleClicked: {
                                var selectedItem = data;
                                var page = listFeedDetailsViewList.createObject();
                                page.pageTitle = selectedItem.data.title;
                                page.postID = (selectedItem.data.name);
                                feedListItem.ListItem.view.pushPageStack(page);
                            }
                        }
                    },
                    ListItemComponent {
                        //id: listItemRoot
                        type: "threadSelf"
                        FeedListItem {
                            id: feedListItemSelf
                            fontsize: feedListItemSelf.ListItem.view.fontSize
                            showSubreddit: true
                            contextActions: [
                                ActionSet {
                                    title: ListItemData.data.title
                                    subtitle: ListItemData.data.author
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("u")
                                            }
                                        ]
                                        id: upvoteActionSelf
                                        title: "Upvote"
                                        imageSource: "asset:///images/upvoteIcon.png"
                                        onTriggered: {
                                            if (feedListItemSelf.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastSelf.body = "You must login to vote!";
                                                accountLoginToastSelf.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == true) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == false) {
                                                    postActionsSelf.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.score += 2;
                                                    feedListItemSelf.ListItem.view.dataModel.replace(feedListItemSelf.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActionsSelf.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.score += 1;
                                                    feedListItemSelf.ListItem.view.dataModel.replace(feedListItemSelf.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("d")
                                            }
                                        ]
                                        id: downvoteActionSelf
                                        title: "Downvote"
                                        imageSource: "asset:///images/downvoteIcon.png"
                                        onTriggered: {
                                            if (feedListItemSelf.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastSelf.body = "You must login to vote!";
                                                accountLoginToastSelf.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == false) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == true) {
                                                    postActionsSelf.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.score -= 2;
                                                    feedListItemSelf.ListItem.view.dataModel.replace(feedListItemSelf.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActionsSelf.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.score -= 1;
                                                    feedListItemSelf.ListItem.view.dataModel.replace(feedListItemSelf.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("c")
                                            }
                                        ]
                                        title: "Comments (" + ListItemData.data.num_comments + ")"
                                        imageSource: "asset:///images/commentsIcon.png"
                                        onTriggered: {
                                            var selectedItem = ListItemData;
                                            var page = listFeedDetailsViewListSelf.createObject();
                                            page.pageTitle = selectedItem.data.title;
                                            page.postID = (selectedItem.data.name);
                                            feedListItemSelf.ListItem.view.pushPageStack(page);
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("s")
                                            }
                                        ]
                                        id: saveActionSelf
                                        title: "Save"
                                        imageSource: "asset:///images/star.png"
                                        onTriggered: {
                                            if (feedListItemSelf.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastSelf.body = "You must login to save posts!";
                                                accountLoginToastSelf.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.saved == false) {
                                                    postActionsSelf.savePost(selectedItem.data.name, "save");
                                                    selectedItem.data.saved = true;
                                                    feedListItemSelf.ListItem.view.dataModel.replace(feedListItemSelf.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActionsSelf.savePost(selectedItem.data.name, "unsave");
                                                    selectedItem.data.saved = false;
                                                    feedListItemSelf.ListItem.view.dataModel.replace(feedListItemSelf.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("h")
                                            }
                                        ]
                                        id: hideActionSelf
                                        title: "Hide"
                                        imageSource: "asset:///images/hide.png"
                                        onTriggered: {
                                            if (feedListItemSelf.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastSelf.body = "You must login to hide posts!";
                                                accountLoginToastSelf.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.hidden == false) {
                                                    postActionsSelf.hidePost(true, selectedItem.data.name);
                                                    feedListItemSelf.ListItem.view.dataModel.removeAt(feedListItemSelf.ListItem.indexPath);
                                                } else {
                                                    postActionsSelf.hidePost(false, selectedItem.data.name);
                                                    feedListItemSelf.ListItem.view.dataModel.removeAt(feedListItemSelf.ListItem.indexPath);
                                                }
                                            }
                                        }
                                    }
                                    InvokeActionItem {
                                        id: shareActionSelf
                                        objectName: "shareActionSelf"
                                        title: "Share"
                                        query {
                                            mimeType: "text/plain"
                                            invokeActionId: "bb.action.SHARE"
                                        }
                                        onTriggered: {
                                            data = ListItemData.data.title + " - https://www.reddit.com" + ListItemData.data.permalink + " via Reddit in Motion";
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("g")
                                            }
                                        ]
                                        id: gotoActionSelf
                                        imageSource: "asset:///images/gotoIcon.png"
                                        objectName: "gotoAction"
                                        title: "Goto /r/" + ListItemData.data.subreddit
                                        //enabled: feedListShowSub
                                        onTriggered: {
                                            var selectedItem = ListItemData;
                                            var page = feedViewGotoSelf.createObject();
                                            var subUrl = selectedItem.data.subreddit;
                                            var firstSlash = subUrl.lastIndexOf("/");
                                            if (firstSlash == -1) {
                                                page.subUrl = "/r/" + subUrl + "/";
                                            } else {
                                                page.subUrl = "/r" + subUrl.substring(firstSlash) + "/";
                                            }

                                            //page.appBackup = app;
                                            feedListItemSelf.ListItem.view.pushPageStack(page);
                                        }
                                    }
                                    DeleteActionItem {
                                        id: deleteActionSelf
                                        onTriggered: {
                                            confirmDeleteSelf.show();
                                            //postActionsComments.deletePost(ListItemData.data.name);
                                            //accountCommentListItem.ListItem.view.dataModel.removeAt(accountCommentListItem.ListItem.indexPath);
                                        }
                                        function deleteItem() {
                                            postActionsSelf.deletePost(ListItemData.data.name);
                                            feedListItemSelf.ListItem.view.dataModel.removeAt(feedListItemSelf.ListItem.indexPath);
                                        }
                                        attachedObjects: [
                                            SystemDialog {
                                                id: confirmDeleteSelf
                                                title: "Delete?"
                                                body: "This will delete the selected item."
                                                onFinished: {
                                                    if (confirmDeleteSelf.result != SystemUiResult.CancelButtonSelection) {
                                                        deleteActionSelf.deleteItem();
                                                    }
                                                }
                                            }
                                        ]
                                    }
                                    attachedObjects: [
                                        SystemToast {
                                            id: accountLoginToastSelf
                                            body: "You must login to vote!"
                                        },
                                        ComponentDefinition {
                                            id: feedViewGotoSelf
                                            source: "FeedView.qml"
                                        }
                                    ]
                                }
                            ]
                            attachedObjects: [
                                ComponentDefinition {
                                    id: listFeedDetailsViewListSelf
                                    source: "FeedDetails.qml"
                                },
                                PostActions {
                                    id: postActionsSelf
                                }
                            ]
                            contextMenuHandler: ContextMenuHandler {
                                onPopulating: {
                                    if (ListItemData.data.likes == true) {
                                        upvoteAction.setImageSource("asset:///images/upvoteSelectedIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    } else if (ListItemData.data.likes == false) {
                                        downvoteAction.setImageSource("asset:///images/downvoteSelectedIcon.png");
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                    } else {
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    }
                                    if (ListItemData.data.saved == true) {
                                        saveAction.setImageSource("asset:///images/saved.png");
                                        saveAction.setTitle("Unsave");
                                    } else {
                                        saveAction.setImageSource("asset:///images/star.png");
                                        saveAction.setTitle("Save");
                                    }
                                    if (ListItemData.data.hidden == true) {
                                        hideAction.setImageSource("asset:///images/unhide.png");
                                        hideAction.setTitle("Unhide");
                                    } else {
                                        hideAction.setImageSource("asset:///images/hide.png");
                                        hideAction.setTitle("Hide");
                                    }
                                }
                            }
                            onCommentBubbleClicked: {
                                var selectedItem = data;
                                var page = listFeedDetailsViewListSelf.createObject();
                                page.pageTitle = selectedItem.data.title;
                                page.postID = (selectedItem.data.name);
                                feedListItemSelf.ListItem.view.pushPageStack(page);
                            }
                        }
                    },
                    ListItemComponent {
                        type: "comment"
                        id: accountCommentListItemRoot
                        CommentListItem {
                            id: accountCommentListItem
                            contextActions: [
                                ActionSet {
                                    title: removeNewLines()
                                    subtitle: ListItemData.data.author
                                    function removeNewLines() {
                                        var noLines = ListItemData.data.body
                                        var intIndexOfMatch = noLines.indexOf("\n");
                                        while (intIndexOfMatch != -1) {
                                            noLines = noLines.replace("\n", " ");
                                            intIndexOfMatch = noLines.indexOf("\n");
                                        }
                                        return noLines;
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("u")
                                            }
                                        ]
                                        id: upvoteActionSubmitted
                                        title: "Upvote"
                                        imageSource: "asset:///images/upvoteIcon.png"
                                        onTriggered: {
                                            if (accountCommentListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastComments.body = "You must login to vote!";
                                                accountLoginToastComments.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == true) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == false) {
                                                    postActionsComments.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.ups += 2;
                                                    accountCommentListItem.ListItem.view.dataModel.replace(accountCommentListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActionsComments.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.ups += 1;
                                                    accountCommentListItem.ListItem.view.dataModel.replace(accountCommentListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("d")
                                            }
                                        ]
                                        id: downvoteActionSubmitted
                                        title: "Downvote"
                                        imageSource: "asset:///images/downvoteIcon.png"
                                        onTriggered: {
                                            if (accountCommentListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastComments.body = "You must login to vote!";
                                                accountLoginToastComments.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == false) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == true) {
                                                    postActionsComments.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.downs += 2;
                                                    accountCommentListItem.ListItem.view.dataModel.replace(accountCommentListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActionsComments.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.downs += 1;
                                                    accountCommentListItem.ListItem.view.dataModel.replace(accountCommentListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("s")
                                            }
                                        ]
                                        id: saveCommentAction
                                        title: "Save"
                                        imageSource: "asset:///images/star.png"
                                        onTriggered: {
                                            if (accountCommentListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastComments.body = "You must login to vote!";
                                                accountLoginToastComments.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.saved == false) {
                                                    postActionsComments.savePost(selectedItem.data.name, "save");
                                                    selectedItem.data.saved = true;
                                                    accountCommentListItem.ListItem.view.dataModel.replace(accountCommentListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActionsComments.savePost(selectedItem.data.name, "unsave");
                                                    selectedItem.data.saved = false;
                                                    accountCommentListItem.ListItem.view.dataModel.replace(accountCommentListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("e")
                                            }
                                        ]
                                        title: "Edit"
                                        id: editAction
                                        imageSource: "asset:///images/ic_edit.png"
                                        onTriggered: {
                                            if (accountCommentListItem.ListItem.view.checkLoggedIn() == false) {
                                                accountLoginToastComments.body = "You must login to edit your post!";
                                                accountLoginToastComments.show();
                                            } else {
                                                accountCommentListItem.ListItem.view.editPost(ListItemData);
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("c")
                                            }
                                        ]
                                        id: copyAction
                                        title: "Copy"
                                        imageSource: "asset:///images/ic_copy_link.png"
                                        onTriggered: {
                                            accountCommentListItem.ListItem.view._copyToClipboard(ListItemData.data.body)
                                        }
                                    }
                                    InvokeActionItem {
                                        id: shareActionComment
                                        objectName: "shareActionComment"
                                        title: "Share"
                                        query {
                                            mimeType: "text/plain"
                                            invokeActionId: "bb.action.SHARE"
                                        }
                                        onTriggered: {
                                            var link_id = ListItemData.data.link_id;
                                            data = "https://www.reddit.com/comments/" + link_id.substr(3) + "/" + ListItemData.data.subreddit + "/" + ListItemData.data.id + " via Reddit in Motion";
                                        }
                                    }
                                    DeleteActionItem {
                                        id: deleteAction
                                        onTriggered: {
                                            confirmDelete.show();
                                            //postActionsComments.deletePost(ListItemData.data.name);
                                            //accountCommentListItem.ListItem.view.dataModel.removeAt(accountCommentListItem.ListItem.indexPath);
                                        }
                                        function deleteItem() {
                                            postActionsComments.deletePost(ListItemData.data.name);
                                            accountCommentListItem.ListItem.view.dataModel.removeAt(accountCommentListItem.ListItem.indexPath);
                                        }
                                        attachedObjects: [
                                            SystemDialog {
                                                id: confirmDelete
                                                title: "Delete?"
                                                body: "This will delete the selected item."
                                                onFinished: {
                                                    if (confirmDelete.result != SystemUiResult.CancelButtonSelection) {
                                                        deleteAction.deleteItem();
                                                    }
                                                }
                                            }
                                        ]
                                    }
                                    attachedObjects: [
                                        SystemToast {
                                            id: accountLoginToastComments
                                            body: "You must login to vote!"
                                        },
                                        PostActions {
                                            id: postActionsComments
                                        }
                                    ]
                                }
                            ]
                            contextMenuHandler: ContextMenuHandler {
                                id: accountContextHandlerComments
                                onPopulating: {
                                    if (ListItemData.data.likes == true) {
                                        upvoteActionSubmitted.setImageSource("asset:///images/upvoteSelectedIcon.png");
                                        downvoteActionSubmitted.setImageSource("asset:///images/downvoteIcon.png");
                                    } else if (ListItemData.data.likes == false) {
                                        downvoteActionSubmitted.setImageSource("asset:///images/downvoteSelectedIcon.png");
                                        upvoteActionSubmitted.setImageSource("asset:///images/upvoteIcon.png");
                                    } else {
                                        upvoteActionSubmitted.setImageSource("asset:///images/upvoteIcon.png");
                                        downvoteActionSubmitted.setImageSource("asset:///images/downvoteIcon.png");
                                    }
                                    if (ListItemData.data.saved == true) {
                                        saveCommentAction.setImageSource("asset:///images/saved.png");
                                        saveCommentAction.setTitle("Unsave");
                                    } else {
                                        saveCommentAction.setImageSource("asset:///images/star.png");
                                        saveCommentAction.setTitle("Save");
                                    }
                                }
                            }
                        }
                    },
                    ListItemComponent {
                        id: footerListItem
                        type: "footer"
                        Container {
                            touchPropagationMode: TouchPropagationMode.None
                            layout: StackLayout {
                                orientation: LayoutOrientation.LeftToRight
                            }
                            id: footerContainer
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            ActivityIndicator {
                                running: true
                                preferredWidth: 100
                                preferredHeight: 100
                                horizontalAlignment: HorizontalAlignment.Center
                                verticalAlignment: VerticalAlignment.Center
                            }
                            Label {
                                verticalAlignment: VerticalAlignment.Center
                                text: "loading more"
                            }
                        }
                    }
                ] //ListItemComponents
                function itemType(data, indexPath) {
                    var currentItem = dataModel.data(indexPath);
                    if (currentItem.kind == "t1") {
                        return "comment";
                    } else if (currentItem.data.type == "footer") {
                        return "footer";
                    } else if (currentItem.kind == "t3") {
                        if (selfPosts == true) {
                            return "threadSelf"
                        } else return "thread"
                    }
                    //from the original non 720x720 file, instead of the above else if statement...
                    /* } else {
                     * return "thread"
                     }*/
                }
                onTriggered: {
                    var page;
                    var selectedItem = dataModel.data(indexPath);
                    if (selectedItem.kind == "t1") {
                        page = feedDetailsViewList.createObject();
                        page.pageTitle = selectedItem.data.link_title;
                        page.postID = selectedItem.data.link_id;
                    } else {
                        var domain = selectedItem.data.domain;
                        var found = domain.search('self');
                        if (found != -1) {
                            page = feedDetailsViewList.createObject();
                            page.pageTitle = selectedItem.data.title;
                            page.postID = selectedItem.data.name;
                        } else {
                            var getOpenLinksBrowser = Qt.app.getSavedValue("openLinksBrowser", "");
                            if (getOpenLinksBrowser == "true") {
                                Qt.app.invokeBrowser(selectedItem.data.url);
                            } else {
                                page = Qt.webView.createObject();
                                page.webUrl = selectedItem.data.url;
                                page.pageTitle = selectedItem.data.title;
                                page.postID = selectedItem.data.name;
                            }
                        }
                        Qt.app.addClicked(selectedItem.data.name);
                        selectedItem.data.clicked = true;
                        dataModel.replace(indexPath, selectedItem);
                    }
                    //navigationPaneTab2.push(page);
                    accountPage.parent.push(page);
                }
                function pushPageStack(pageParam) {
                    navigationPaneTab2.push(pageParam);
                }
                function checkLoggedIn() {
                    return app.getLoggedIn();
                }
                function _copyToClipboard(pText) {
                    Qt.app.copyToClipboard(pText);
                }
                function editPost(selectedItem) {
                    editSheetOut.author = selectedItem.data.author;
                    if (selectedItem.data.likes != undefined) {
                        editSheetOut.likes = selectedItem.data.likes;
                    }
                    editSheetOut.body = selectedItem.data.body;
                    editSheetOut.name = selectedItem.data.name;
                    editSheetOut.votes = (selectedItem.data.ups - selectedItem.data.downs);
                    editSheetOut.subject = "Edit Comment";

                    editSheet.open();
                }
                function getFontsize() {
                    var value;
                    if (Qt.app) {
                        value = Qt.app.getSavedValue("fontsize", "");
                    } else {
                        value = app.getSavedValue("fontsize", "");
                    }
                    if (value == "Large") {
                        return FontSize.Large;
                    } else if (value == "Medium") {
                        return FontSize.Medium;
                    } else return FontSize.Small;
                }
                onDataModelChanged: {
                    fontSize = getFontsize();
                    if (dataModel.size() <= 0) {
                        allowAtEndSignal = false;
                        emptyLabel.setVisible(true);
                    } else {
                        allowAtEndSignal = true;
                        emptyLabel.setVisible(false);
                    }
                }
                attachedObjects: [
                    /*ComponentDefinition {
                     * id: webView
                     * source: "PostWebView.qml"
                     * },*/
                    ComponentDefinition {
                        id: feedDetailsViewList
                        source: "FeedDetails.qml"
                    },
                    ListScrollStateHandler {
                        onAtEndChanged: {
                            if (atEnd == true && allowAtEndSignal == true) {
                                allowAtEndSignal = false;
                                feedHttp.getNextPageUser(accountBar.selectedTab, accountList);
                            }
                        }
                    },
                    Sheet {
                        id: editSheet
                        EditSheet {
                            id: editSheetOut
                            onBack: {
                                editSheet.close();
                            }
                        }
                    }
                ]
            }
            ActivityIndicator {
                id: accountActivity
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                preferredWidth: 200
                preferredHeight: 200
            }
            Label {
                id: emptyLabel
                visible: false
                text: "there doesn't seem to be anything here"
                textStyle.color: Color.Red
                multiline: true
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                leftMargin: 20
                rightMargin: 20
            }
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onNextPageLoaded: {
                allowAtEndSignal = true;
            }
            onUserInfoRetrieved: {
                titlebarContainer.usernameP = username;
                titlebarContainer.commentKarmaP = comment_karma;
                titlebarContainer.linkKarmaP = link_karma;
            }
        }
    ]
    onCreationCompleted: {
        accountActivity.start();
        feedHttp.getUserInfo();
    }
}