import bb.cascades 1.2
import org.labsquare 1.0
import "markdown2html.js" as MarkdownConverter

Container {
    signal openWebview(string pUrl)
    visible: getVisible()
    horizontalAlignment: HorizontalAlignment.Fill
    topMargin: 0
    id: feedItemContainer
    leftPadding: checkDepth()
    property string op
    property int collapseRoot: getCollapseRoot()
    //property string itemUrl: "http://www.reddit.com/comments" + ListItemData.data.link_id + "/" + ListItemData.data.subreddit
    property variant usersVote: getVote()
    property bool isOp: checkOp()
    property variant downVoteColor: Color.create("#6F72FB")
    property variant upVoteColor: Color.create("#F96E3B")
    function getVote() {
        if (ListItemData.data.likes == true) {
            return upVoteColor;
        } else if (ListItemData.data.likes == false) {
            return downVoteColor;
        } else {
            return Color.Black;
        }
    }
    function checkDepth() {
        var depth = ListItemData.depth;
        if (depth == null) {
            return 0;
        } else {
            return depth * 30;
        }
    }
    function getVisible() {
        if (ListItemData.collapse == "true") {
            return false;
        } else return true
    }
    function getCollapseRoot() {
        if (ListItemData.collapseRoot != undefined) {
            if (ListItemData.collapseRoot > 0) {
                itemTitle.setVisible(false);
                collapsedLabel.setText("Double tap to expand (" + ListItemData.collapseRoot + " children)");
                collapsedLabel.setVisible(true);
            } else {
                itemTitle.setVisible(true);
                collapsedLabel.setVisible(false);
            }
            return ListItemData.collapseRoot
        } else {
            return 0;
        }
    }
    function checkOp() {
        if (ListItemData.data.author == op) {
            return true;
        } else {
            return false;
        }
    }
    Container {
        overlapTouchPolicy: OverlapTouchPolicy.Allow
        touchPropagationMode: TouchPropagationMode.PassThrough
        id: outer
        horizontalAlignment: HorizontalAlignment.Fill
        topPadding: 5
        bottomPadding: 5
        rightPadding: 5
        leftPadding: 5
        Container {
            overlapTouchPolicy: OverlapTouchPolicy.Allow
            touchPropagationMode: TouchPropagationMode.PassThrough
            layout: DockLayout {
            }
            Container {
                overlapTouchPolicy: OverlapTouchPolicy.Allow
                touchPropagationMode: TouchPropagationMode.PassThrough
                horizontalAlignment: HorizontalAlignment.Left
                leftPadding: 0
                leftMargin: 0
                rightPadding: 0
                verticalAlignment: VerticalAlignment.Fill
                preferredWidth: 5
                minWidth: 5
                background: getBandColor() //Color.create(getBandColor())
                function getBandColor() {
                    if (ListItemData.depth) {
                        return Color.create(ListItemData.depth * 0.16, 0.5, 1.0, 1.0)
                    } else return Color.create('#00CC00');
                }
            }
            Container {
                overlapTouchPolicy: OverlapTouchPolicy.Allow
                touchPropagationMode: TouchPropagationMode.PassThrough
                id: inner
                horizontalAlignment: HorizontalAlignment.Fill
                topPadding: 10
                rightPadding: 10
                leftPadding: 10
                bottomPadding: 10
                Container {
                    overlapTouchPolicy: OverlapTouchPolicy.Allow
                    touchPropagationMode: TouchPropagationMode.PassThrough
                    leftPadding: 3
                    horizontalAlignment: HorizontalAlignment.Fill
                    Label {
                        activeTextHandler: ActiveTextHandler {
                            id: activeTextHandler
                            onTriggered: {
                                var getOpenLinksBrowser = Qt.app.getSavedValue("openLinksBrowser", "");
                                if (getOpenLinksBrowser != "true") {
                                    event.abort();
                                    if ((event.text).indexOf("www.reddit.") != -1) {
                                        Qt.app.invokeMyself(event.text);
                                    } else {
                                        openWebview(event.text);
                                    }
                                }
                            }
                        }
                        id: itemTitle
                        text: getTitle()
                        content.flags: TextContentFlag.ActiveText
                        textStyle.fontSize: FontSize.Medium
                        multiline: true
                        function getTitle() {
                            if (ListItemData.data.title == null) {
                                var body = MarkdownConverter._RunSpanGamut(ListItemData.data.body);
                                return "<html>" + body + "</html>"
                            } else {
                                return "<html>" + ListItemData.data.title + "</html>";
                            }
                        }
                    }
                    Label {
                        id: collapsedLabel
                        visible: false
                        textStyle.fontSize: FontSize.Medium
                        textStyle.fontStyle: FontStyle.Italic
                        text: "Double tap to expand (0 children)"
                        overlapTouchPolicy: OverlapTouchPolicy.Allow
                    }
                    Container {
                        layout: DockLayout {

                        }
                        horizontalAlignment: HorizontalAlignment.Fill
                        overlapTouchPolicy: OverlapTouchPolicy.Allow
                        Container {
                            overlapTouchPolicy: OverlapTouchPolicy.Allow
                            layout: StackLayout {
                                orientation: LayoutOrientation.LeftToRight
                            }
                            horizontalAlignment: HorizontalAlignment.Left
                            rightPadding: (gildedContainer.visible) ? 75 : 10
                            Label {
                                layoutProperties: StackLayoutProperties {
                                    spaceQuota: -1
                                }
                                overlapTouchPolicy: OverlapTouchPolicy.Allow
                                id: scoreLabel
                                text: (ListItemData.data.ups - ListItemData.data.downs)
                                textStyle.fontSize: FontSize.Medium
                                textStyle.color: usersVote
                                rightMargin: 10
                            }
                            Container {
                                layout: StackLayout {
                                    orientation: LayoutOrientation.LeftToRight
                                }
                                layoutProperties: StackLayoutProperties {
                                    spaceQuota: 1
                                }
                                overlapTouchPolicy: OverlapTouchPolicy.Allow
                                Label {
                                    overlapTouchPolicy: OverlapTouchPolicy.Allow
                                    id: authorLabel
                                    text: (isOp) ? ("by  " + ListItemData.data.author + " [S]") : ("by  " + ListItemData.data.author) //checkOp()
                                    textStyle.fontSize: FontSize.Medium
                                    textStyle.color: (isOp) ? Color.Magenta : Color.DarkGray
                                }
                                Label {
                                    overlapTouchPolicy: OverlapTouchPolicy.Allow
                                    id: timeLabel
                                    text: milisecondsToString(ListItemData.data.created_utc)
                                    textStyle.fontSize: FontSize.Medium
                                    textStyle.color: Color.DarkGray
                                    function milisecondsToString(miliseconds) {
                                        var inputSeconds = parseInt(miliseconds);
                                        var unixtime = inputSeconds * 1000;
                                        var diff = new Date();
                                        var unixtime2 = new Date(unixtime);

                                        var secDiff = (diff.getSeconds() - unixtime2.getSeconds());
                                        var minDiff = (diff.getMinutes() - unixtime2.getMinutes());
                                        var hourDiff = (diff.getHours() - unixtime2.getHours());
                                        var dayDiff = (diff.getDate() - unixtime2.getDate());
                                        var monthDiff = (diff.getMonth() - unixtime2.getMonth());
                                        var yearDiff = (diff.getFullYear() - unixtime2.getFullYear());
                                        var months = [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ];
                                        if (secDiff < 0) {
                                            minDiff --;
                                            secDiff += 60;
                                        }
                                        if (minDiff < 0) {
                                            hourDiff --;
                                            minDiff += 60;
                                        }
                                        if (hourDiff < 0) {
                                            dayDiff --;
                                            hourDiff += 24;
                                        }
                                        if (dayDiff < 0) {
                                            monthDiff --;
                                            dayDiff += months[diff.getMonth() - 2]; // -2 as the months array is zero-indexed too
                                        }
                                        if (monthDiff < 0) {
                                            yearDiff --;
                                            monthDiff += 12;
                                        }
                                        var digit = 0;
                                        var timePeriod = "";

                                        //check and return the largest value of date time initialized.
                                        if (yearDiff > 0) {
                                            digit = yearDiff;
                                            timePeriod = "y";
                                        } else if (monthDiff > 0) {
                                            digit = monthDiff;
                                            timePeriod = "M";
                                        } else if (dayDiff != 0) {
                                            digit = dayDiff;
                                            timePeriod = "d";
                                        } else if (hourDiff != 0) {
                                            digit = hourDiff;
                                            timePeriod = "h";
                                        } else if (minDiff != 0) {
                                            digit = minDiff;
                                            timePeriod = "m";
                                        } else if (secDiff != 0) {
                                            digit = secDiff;
                                            timePeriod = "s";
                                        }
                                        return digit + timePeriod + " ago";
                                    }
                                }
                            }
                        }
                        Container {
                            id: gildedContainer
                            overlapTouchPolicy: OverlapTouchPolicy.Allow
                            horizontalAlignment: HorizontalAlignment.Right
                            verticalAlignment: VerticalAlignment.Center
                            //rightPadding: 10
                            leftPadding: 10
                            visible: (ListItemData.data.gilded > 0) ? true : false
                            Container {
                                overlapTouchPolicy: OverlapTouchPolicy.Allow
                                layout: StackLayout {
                                    orientation: LayoutOrientation.LeftToRight
                                }
                                Label {
                                    overlapTouchPolicy: OverlapTouchPolicy.Allow
                                    rightMargin: 0
                                    id: gildedLabel
                                    //textStyle.textAlign: TextAlign.Right
                                    textStyle.fontSize: FontSize.Medium
                                    textStyle.color: Color.DarkGray
                                    visible: (ListItemData.data.gilded > 1) ? true : false
                                    text: "x" + ListItemData.data.gilded
                                }
                                ImageView {
                                    overlapTouchPolicy: OverlapTouchPolicy.Allow
                                    verticalAlignment: VerticalAlignment.Fill
                                    //horizontalAlignment: HorizontalAlignment.Fill
                                    leftMargin: 0
                                    topMargin: 0
                                    bottomMargin: 0
                                    id: goldIndicatorImage
                                    imageSource: "asset:///images/goldIndicator.png"
                                    scalingMethod: ScalingMethod.AspectFit
                                    minHeight: 40
                                    minWidth: 40
                                    maxHeight: 40
                                    maxWidth: 40
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    Container {
        overlapTouchPolicy: OverlapTouchPolicy.Allow
        touchPropagationMode: TouchPropagationMode.PassThrough
        background: Color.LightGray
        preferredHeight: 1
        preferredWidth: Infinity
        topPadding: 0
        bottomPadding: 0
        topMargin: 0
        bottomMargin: 0
    }
    property variant blueBackground: Color.create("#00A7DD")
    gestureHandlers: [
        LongPressHandler {
            onLongPressed: {
                outer.background = blueBackground
                inner.background = blueBackground
            }
        }
    ]
    onTouch: {
        if (event.touchType == TouchType.Down) {
            outer.background = blueBackground
            inner.background = Color.White
        } else if (event.touchType == TouchType.Up || event.touchType == TouchType.Cancel) {
            inner.background = Color.Transparent;
            outer.background = Color.Transparent;
        }
    }
}