import bb.cascades 1.0
import bb.system 1.0
import org.labsquare 1.0
import Network.MessagesHttp 1.0

Page {
    objectName: "composeMessageSheet"
    id: composeMessageSheet
    property string captchaIden: "captchaIdenTemp"
    signal back()
    signal sendMessage()
    titleBar: TitleBar {
        title: "Compose"
        dismissAction: ActionItem {
            title: "Cancel"
            onTriggered: {
                // Cancel has been pressed so we emit the cancel signal (will hide the sheet).
                composeMessageSheet.back();
            }
        }
        acceptAction: ActionItem {
            title: "Send"
            id: acceptAction
            onTriggered: {
                if (toField.text != "" && subjectField.text != "" && messageField.text != "") {
                    activity.start();
                    acceptAction.setEnabled(false);
                    messagesHttp.composeMessage(toField.text, subjectField.text, messageField.text, captchaText.text, captchaIden);
                } else {
                    emptyFieldErrorToast.show();
                }
            }
        }
        attachedObjects: [
            SystemToast {
                id: emptyFieldErrorToast
                body: "One or more fields have not been filled..."
            }
        ]
    }
    Container {
        layout: DockLayout {

        }
        ScrollView {
            Container {
                topPadding: 20
                leftPadding: 10
                rightPadding: 10
                TextField {
                    id: toField
                    hintText: "to (username)"
                }
                Divider {

                }
                TextField {
                    id: subjectField
                    hintText: "subject"

                }
                Divider {

                }
                TextArea {
                    id: messageField
                    hintText: "message"
                    preferredHeight: 200
                }
                Container {
                    id: captchaContainer
                    visible: false
                    Divider {
                    }
                    WebImageView {
                        id: captchaImage
                        objectName: "captchaImage"
                        preferredHeight: 200
                        preferredWidth: 748
                        scalingMethod: ScalingMethod.AspectFit
                    }
                    TextField {
                        id: captchaText
                        hintText: "human?"
                    }
                }
            }
        }
        ActivityIndicator {
            id: activity
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
    }
    attachedObjects: [
        MessagesHttp {
            id: messagesHttp
            onComposeMessageComplete: {
                messageSendError.body = error;
                if (error == "your message has been delivered") {
                    composeMessageSheet.sendMessage();
                    toField.resetText();
                    subjectField.resetText();
                    messageField.resetText();
                } else if (error == "care to try these again?") {
                    captchaContainer.visible = true;
                    messageSendError.body = "Complete the captcha to continue"
                }
                acceptAction.setEnabled(true);
                activity.stop();
                messageSendError.show();
            }
            onCaptchaUrlRetrieved: {
                captchaIden = captchaIdenParam;
            }
        },
        SystemToast {
            id: messageSendError
        }

    ]
    onCaptchaIdenChanged: {
        captchaImage.url = "https://www.reddit.com/captcha/" + captchaIden + ".png";
    }
}
