import bb.cascades 1.0
import bb.system 1.0
import org.labsquare 1.0
import Network.MessagesHttp 1.0

Page {
    id: composeThreadSheet
    property string captchaIden: "captchaIdenTemp"
    property string currentSub: ""
    property string contentValue
    signal back()
    signal postThread()
    titleBar: TitleBar {
        title: "Submit to Reddit"
        dismissAction: ActionItem {
            title: "Cancel"
            onTriggered: {
                // Cancel has been pressed so we emit the cancel signal (will hide the sheet).
                composeThreadSheet.back();
            }
        }
        acceptAction: ActionItem {
            title: "Submit"
            id: acceptAction
            onTriggered: {
                if (segControl.selectedIndex == 0) {
                    if (titleField.text != "" && contentField.text != "" && subredditField.text != "") {
                        activity.start();
                        acceptAction.setEnabled(false);
                        messagesHttp.submitPost(titleField.text, contentField.text, subredditField.text, captchaText.text, captchaIden, segControl.selectedOption.description, sendRepliesCheckbox.checked);
                    } else {
                        emptyFieldErrorToast.show();
                    }
                } else {
                    if (titleField.text != "" && subredditField.text != "") {
                        activity.start();
                        acceptAction.setEnabled(false);
                        messagesHttp.submitPost(titleField.text, contentField.text, subredditField.text, captchaText.text, captchaIden, segControl.selectedOption.description, sendRepliesCheckbox.checked);
                    } else {
                        emptyFieldErrorToast.show();
                    }
                }
            }
        }
        attachedObjects: [
            SystemToast {
                id: emptyFieldErrorToast
                body: "One or more fields have not been filled..."
            }
        ]
    }
    Container {
        layout: DockLayout {

        }
        ScrollView {
            Container {
                topPadding: 20
                leftPadding: 10
                rightPadding: 10
                SegmentedControl {
                    id: segControl
                    options: [
                        Option {
                            text: "Link"
                            description: "link"
                            selected: true
                        },
                        Option {
                            text: "Text"
                            description: "self"
                        }
                    ]
                    onSelectedIndexChanged: {
                        if (selectedIndex == 0) {
                            contentField.setHintText("url");
                            contentField.resetMinHeight();
                        } else {
                            contentField.setHintText("text (optional)");
                            contentField.setMinHeight(200);
                        }
                    }
                }
                TextArea {
                    id: titleField
                    hintText: "title"
                }
                Divider {

                }
                TextArea {
                    id: contentField
                    hintText: "url"

                }
                Divider {

                }
                TextField {
                    id: subredditField
                    hintText: "choose a subreddit"
                    text: currentSub
                }
                Divider {

                }
                Container {
                    leftPadding: 20
                    rightPadding: 20
                    CheckBox {
                        id: sendRepliesCheckbox
                        text: "send replies to my inbox"
                        checked: false
                    }
                }
                Container {
                    id: captchaContainer
                    visible: false
                    Divider {
                    }
                    WebImageView {
                        id: captchaImage
                        objectName: "captchaImage"
                        preferredHeight: 200
                        preferredWidth: 748
                        scalingMethod: ScalingMethod.AspectFit
                    }
                    TextField {
                        id: captchaText
                        hintText: "human?"
                    }
                }
            }
        }
        ActivityIndicator {
            id: activity
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
    }
    attachedObjects: [
        MessagesHttp {
            id: messagesHttp
            onComposeMessageComplete: {
                messageSendError.body = error;
                if (error == "post successful!") {
                    composeThreadSheet.postThread();
                    titleField.resetText();
                    contentField.resetText();
                    subredditField.resetText();
                } else if (error == "care to try these again?") {
                    captchaContainer.visible = true;
                    messageSendError.body = "Complete the captcha to continue"
                }
                acceptAction.setEnabled(true);
                activity.stop();
                messageSendError.show();
            }
            onCaptchaUrlRetrieved: {
                captchaIden = captchaIdenParam;
            }
        },
        SystemToast {
            id: messageSendError
        }

    ]
    onCaptchaIdenChanged: {
        captchaImage.url = "https://www.reddit.com/captcha/" + captchaIden + ".png";
    }
    onContentValueChanged: {
        contentField.setText(contentValue);
    }
}
