import bb.cascades 1.2
import bb.system 1.2
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0
import "markdown2html.js" as MarkdownConverter

Page {
    id: feedDetails
    property string pageTitle
    property string postID
    property string selfText
    property int score
    property string author
    property string likes
    property string permalink
    property bool allowAtEndSignal
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    titleBar: TitleBar {
        title: pageTitle
    }
    actions: [
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("c")
                }
            ]
            id: composeAction
            title: "Compose"
            enabled: false
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/add_comment.png"
            onTriggered: {
                if (postActionsCompose.getLoggedIn() == false) {
                    loginToastCompose.body = "You must login to comment!";
                    loginToastCompose.show();
                } else {
                    commentReplySheetOut.author = author;
                    if (likes != "") {
                        commentReplySheetOut.likes = likes;
                    }
                    commentReplySheetOut.body = pageTitle;
                    commentReplySheetOut.name = postID;
                    commentReplySheetOut.votes = score;
                    commentReplySheetOut.subject = "Reply to Thread";
                    commentReplySheet.open();
                }
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("e")
                }
            ]
            title: "Edit"
            imageSource: "asset:///images/ic_edit.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            enabled: getEditEnabled()
            onTriggered: {
                if (postActionsCompose.getLoggedIn() == false) {
                    loginToastCompose.body = "You must login to edit posts!";
                    loginToastCompose.show();
                } else {
                    editSheetOut.author = feedDetails.author;
                    if (feedDetails.likes != undefined) {
                        editSheetOut.likes = feedDetails.likes;
                    }
                    editSheetOut.body = feedDetails.selfText
                    editSheetOut.name = feedDetails.postID
                    editSheetOut.votes = feedDetails.score
                    editSheetOut.subject = "Edit Post";

                    editSheet.open();
                }
            }
            function getEditEnabled() {
                if (feedDetails.selfText != "" && feedDetails.author.toLowerCase() == Qt.app.getSavedUsername().toLowerCase())
                    return true;
                else return false;
            }
        },
        InvokeActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("o")
                }
            ]
            id: invokeB
            title: "Open in Browser"
            //imageSource: "images/globe.png"
            query {
                uri: "https://www.reddit.com" + permalink
                mimeType: "text/html"
                invokeActionId: "bb.action.OPEN"
                onQueryChanged: invokeB.query.updateQuery()
            }
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToTop
                }
            ]
            title: "To Top"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_top.png"
            onTriggered: {
                commentList.scrollToItem([ 0 ]);
            }
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToBottom
                }
            ]
            title: "To Bottom"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_bottom.png"
            onTriggered: {
                commentList.scrollToItem([ commentList.dataModel.size() - 1 ]);
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("r")
                }
            ]
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: "Refresh"
            imageSource: "asset:///images/reloadIcon.png"
            onTriggered: {
                feedHttp.getComments(postID, commentsIndicator, commentList, sortedDropdown.selectedOption.text);
            }
        }
    ]
    Container {
        Container {
            layout: DockLayout {
            }
            ListView {
                scrollRole: ScrollRole.Main
                property int leadingHeight: 0
                property string opAuthor: author
                objectName: "commentList"
                id: commentList
                leadingVisual: Container {
                    id: leadingContainer
                    horizontalAlignment: HorizontalAlignment.Center
                    background: Color.create("#CEE3F8")
                    DropDown {
                        id: sortedDropdown
                        preferredWidth: Infinity
                        horizontalAlignment: HorizontalAlignment.Center
                        bottomMargin: 10
                        title: "sorted by:"
                        options: [
                            Option {
                                text: "best"
                                description: "confidence"
                                selected: true
                            },
                            Option {
                                text: "top"
                                description: "top"
                            },
                            Option {
                                text: "new"
                                description: "new"
                            },
                            Option {
                                text: "hot"
                                description: "hot"
                            },
                            Option {
                                text: "controversial"
                                description: "controversial"
                            },
                            Option {
                                text: "old"
                                description: "old"
                            }
                        ]
                        onSelectedOptionChanged: {
                            feedHttp.getComments(postID, commentsIndicator, commentList, selectedOption.description);
                        }
                    }
                }
                function setInfoVisible() {
                    if (selfText == "") {
                        return false
                    } else {
                        return true
                    }
                }
                listItemComponents: [
                    ListItemComponent {
                        type: "header"
                        Container {
                            leftPadding: 10
                            rightPadding: 10
                            id: selfTextContainer
                            horizontalAlignment: HorizontalAlignment.Fill
                            preferredWidth: Infinity
                            background: Color.create("#CEE3F8")
                            Label {
                                activeTextHandler: ActiveTextHandler {
                                    id: activeTextHandler
                                    onTriggered: {
                                        var getOpenLinksBrowser = Qt.app.getSavedValue("openLinksBrowser", "");
                                        if (getOpenLinksBrowser != "true") {
                                            event.abort();
                                            if ((event.text).indexOf("www.reddit.") != -1) {
                                                Qt.app.invokeMyself(event.text);
                                            } else {
                                                selfTextContainer.ListItem.view.openWebview(event.text);
                                            }
                                        }
                                    }
                                }
                                id: leadingInfoLabel
                                objectName: "leadingInfoLabel"
                                text: checkSelftext()
                                multiline: true
                                textStyle.color: Color.Black
                                content.flags: TextContentFlag.ActiveText
                                function checkSelftext() {
                                    if (ListItemData.data.selftext != "") {
                                        var formattedText = MarkdownConverter._RunSpanGamut(ListItemData.data.selftext);
                                        return "<html>" + formattedText + "</html>";
                                    } else {
                                        leadingInfoLabel.setVisible(false);
                                        return "";
                                    }
                                }
                            }
                            Container {
                                layout: DockLayout {

                                }
                                horizontalAlignment: HorizontalAlignment.Fill
                                Container {
                                    layout: StackLayout {
                                        orientation: LayoutOrientation.LeftToRight
                                    }
                                    horizontalAlignment: HorizontalAlignment.Left
                                    rightPadding: (gildedContainer.visible) ? 75 : 10
                                    Label {
                                        id: scoreLabel
                                        text: ListItemData.data.score
                                        textStyle.fontSize: FontSize.Medium
                                        textStyle.color: getVote()
                                        rightMargin: 10
                                        function getVote() {
                                            if (ListItemData.data.likes == "true") {
                                                return Color.create("#F96E3B");
                                            } else if (ListItemData.data.likes == "false") {
                                                return Color.create("#6F72FB");
                                            } else {
                                                return Color.Black
                                            }
                                        }
                                    }
                                    Label {
                                        id: authorLabel
                                        text: "by " + ListItemData.data.author + " to " + ListItemData.data.subreddit
                                        textStyle.fontSize: FontSize.Medium
                                        textStyle.color: Color.DarkGray
                                        touchPropagationMode: TouchPropagationMode.PassThrough
                                    }
                                }
                                Container {
                                    id: gildedContainer
                                    horizontalAlignment: HorizontalAlignment.Right
                                    verticalAlignment: VerticalAlignment.Center
                                    //rightPadding: 10
                                    leftPadding: 10
                                    visible: (ListItemData.data.gilded > 0) ? true : false
                                    Container {
                                        layout: StackLayout {
                                            orientation: LayoutOrientation.LeftToRight
                                        }
                                        Label {
                                            rightMargin: 0
                                            id: gildedLabel
                                            //textStyle.textAlign: TextAlign.Right
                                            textStyle.fontSize: FontSize.Medium
                                            textStyle.color: Color.DarkGray
                                            visible: (ListItemData.data.gilded > 1) ? true : false
                                            text: "x" + ListItemData.data.gilded
                                        }
                                        ImageView {
                                            verticalAlignment: VerticalAlignment.Fill
                                            //horizontalAlignment: HorizontalAlignment.Fill
                                            leftMargin: 0
                                            topMargin: 0
                                            bottomMargin: 0
                                            id: goldIndicatorImage
                                            imageSource: "asset:///images/goldIndicator.png"
                                            scalingMethod: ScalingMethod.AspectFit
                                            minHeight: 40
                                            minWidth: 40
                                            maxHeight: 40
                                            maxWidth: 40
                                        }
                                    }
                                }
                            }
                        }
                    },
                    ListItemComponent {
                        type: "more"
                        Container {
                            visible: false
                        }
                    },
                    ListItemComponent {
                        type: "item"
                        CommentListItem {
                            op: commentListItem.ListItem.view.opAuthor
                            id: commentListItem
                            onOpenWebview: {
                                commentListItem.ListItem.view.openWebview(pUrl);
                            }
                            gestureHandlers: DoubleTapHandler {
                                onDoubleTapped: {
                                    var selectedItem = ListItemData;
                                    commentListItem.ListItem.view.collapseComments(selectedItem, commentListItem.ListItem.indexPath);
                                }
                            }
                            contextActions: [
                                ActionSet {
                                    title: removeNewLines()
                                    subtitle: ListItemData.data.author
                                    function removeNewLines() {
                                        var noLines = ListItemData.data.body
                                        var intIndexOfMatch = noLines.indexOf("\n");
                                        while (intIndexOfMatch != -1) {
                                            noLines = noLines.replace("\n", " ");
                                            intIndexOfMatch = noLines.indexOf("\n");
                                        }
                                        return noLines;
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("u")
                                            }
                                        ]
                                        id: upvoteAction
                                        title: "Upvote"
                                        imageSource: "asset:///images/upvoteIcon.png"
                                        onTriggered: {
                                            if (postActions.getLoggedIn() == false) {
                                                loginToast.body = "You must login to vote!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == true) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == false) {
                                                    postActions.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.ups += 2;
                                                    commentListItem.ListItem.view.dataModel.replace(commentListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActions.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.ups += 1;
                                                    commentListItem.ListItem.view.dataModel.replace(commentListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("d")
                                            }
                                        ]
                                        id: downvoteAction
                                        title: "Downvote"
                                        imageSource: "asset:///images/downvoteIcon.png"
                                        onTriggered: {
                                            if (postActions.getLoggedIn() == false) {
                                                loginToast.body = "You must login to vote!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == false) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == true) {
                                                    postActions.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.downs += 2;
                                                    commentListItem.ListItem.view.dataModel.replace(commentListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActions.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.downs += 1;
                                                    commentListItem.ListItem.view.dataModel.replace(commentListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("s")
                                            }
                                        ]
                                        id: saveAction
                                        title: "Save"
                                        imageSource: "asset:///images/star.png"
                                        onTriggered: {
                                            if (postActions.getLoggedIn() == false) {
                                                loginToast.body = "You must login to save posts!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.saved == false) {
                                                    postActions.savePost(selectedItem.data.name, "save");
                                                    selectedItem.data.saved = true;
                                                    commentListItem.ListItem.view.dataModel.replace(commentListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    postActions.savePost(selectedItem.data.name, "unsave");
                                                    selectedItem.data.saved = false;
                                                    commentListItem.ListItem.view.dataModel.replace(commentListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("r")
                                            }
                                        ]
                                        id: replyAction
                                        title: "Reply"
                                        imageSource: "asset:///images/ic_reply_as_email.png"
                                        onTriggered: {
                                            if (postActions.getLoggedIn() == false) {
                                                loginToast.body = "You must login to reply!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                commentListItem.ListItem.view.pushReplyComment(selectedItem);
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("c")
                                            }
                                        ]
                                        id: copyAction
                                        title: "Copy"
                                        imageSource: "asset:///images/ic_copy_link.png"
                                        onTriggered: {
                                            commentListItem.ListItem.view._copyToClipboard(ListItemData.data.body)
                                        }
                                    }
                                    InvokeActionItem {
                                        id: shareAction
                                        objectName: "shareAction"
                                        title: "Share"
                                        query {
                                            mimeType: "text/plain"
                                            invokeActionId: "bb.action.SHARE"
                                        }
                                        onTriggered: {
                                            //var link_id = ListItemData.data.link_id;
                                            //data = "http://www.reddit.com/comments/" + link_id.substr(3) + "/" + ListItemData.data.subreddit + "/" + ListItemData.data.id + " via Reddit in Motion";
                                            data = "https://www.reddit.com" + commentListItem.ListItem.view.getPerm() + ListItemData.data.id + " via Reddit in Motion";
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("e")
                                            }
                                        ]
                                        id: collapseAction
                                        title: "Collapse"
                                        imageSource: "asset:///images/collapse.png"
                                        onTriggered: {
                                            var selectedItem = ListItemData;
                                            commentListItem.ListItem.view.collapseComments(selectedItem, commentListItem.ListItem.indexPath);
                                        }
                                    }
                                }
                            ]
                            attachedObjects: [
                                PostActions {
                                    id: postActions
                                },
                                SystemToast {
                                    id: loginToast
                                    body: "You must login to vote!"
                                }
                            ]
                            contextMenuHandler: ContextMenuHandler {
                                id: contextHandler
                                onPopulating: {
                                    if (ListItemData.data.likes == true) {
                                        upvoteAction.setImageSource("asset:///images/upvoteSelectedIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    } else if (ListItemData.data.likes == false) {
                                        downvoteAction.setImageSource("asset:///images/downvoteSelectedIcon.png");
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                    } else {
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    }
                                    if (ListItemData.collapseRoot > 0) {
                                        collapseAction.setImageSource("asset:///images/expand.png");
                                        collapseAction.setTitle("Expand (Double-Tap)");
                                    } else {
                                        collapseAction.setImageSource("asset:///images/collapse.png");
                                        collapseAction.setTitle("Collapse (Double-Tap)");
                                    }
                                    if (ListItemData.data.saved == true) {
                                        saveAction.setImageSource("asset:///images/saved.png");
                                        saveAction.setTitle("Unsave");
                                    } else {
                                        saveAction.setImageSource("asset:///images/star.png");
                                        saveAction.setTitle("Save");
                                    }
                                }
                            }
                        }
                    }
                ] //ListItemComponents
                function pushPageStack(pageParam) {
                    navigationPane.push(pageParam);
                }
                function pushReplyComment(selectedItem) {
                    commentReplySheetOut.author = selectedItem.data.author;
                    if (selectedItem.data.likes != undefined) {
                        commentReplySheetOut.likes = selectedItem.data.likes;
                    }
                    commentReplySheetOut.body = selectedItem.data.body;
                    commentReplySheetOut.name = selectedItem.data.name;
                    commentReplySheetOut.votes = (selectedItem.data.ups - selectedItem.data.downs);
                    commentReplySheetOut.subject = "Reply to Comment";
                    commentReplySheet.open();
                }
                function collapseComments(selectedItem, index) {
                    if (selectedItem.collapseRoot > 0) {
                        Qt.app.expandComments(commentList, selectedItem.depth, index);
                    } else {
                        Qt.app.collapseComments(commentList, selectedItem.depth, index);
                    }
                }
                function _copyToClipboard(pText) {
                    Qt.app.copyToClipboard(pText);
                }
                function itemType(data, indexPath) {
                    var currentItem = dataModel.data(indexPath);
                    if (currentItem.kind == "more") {
                        return "more";
                    } else if (currentItem.kind == "t1") {
                        return "item"
                    } else {
                        return "header"
                    }
                }
                function openWebview(pUrl) {
                    var page = Qt.webView.createObject();
                    page.webUrl = pUrl;
                    page.pageTitle = ""
                    feedDetails.parent.push(page);
                }
                function getPerm() {
                    return feedDetails.permalink;
                }
                onDataModelChanged: {
                    if (selfText != "") {
                        //commentList.scroll(-1);
                        emptyLabel.setVisible(false);
                    } else if (dataModel.size() == 0) {
                        emptyLabel.setVisible(true);
                        allowAtEndSignal = false;
                    } else {
                        emptyLabel.setVisible(false);
                    }
                    if (dataModel.size() > 0) {
                        allowAtEndSignal = true;
                    }
                }
                onLeadingHeightChanged: { //is this necessary any more? maybe is from back when tried to auto scroll up to the selftext since it was a leadingitem
                    commentList.scroll(- (leadingHeight));
                }
                attachedObjects: [
                    ListScrollStateHandler {
                        /*onAtEndChanged: {
                         * if (atEnd == true && allowAtEndSignal == true) {
                         * allowAtEndSignal = false;
                         * feedHttp.getNextPage(titleBar.selectedOption.description, subUrl, feedList);
                         * }
                         }**/
                    }
                ]
            }
            ActivityIndicator {
                objectName: "commentsIndicator"
                id: commentsIndicator
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                preferredWidth: 200
                preferredHeight: 200
            }
            Label {
                id: emptyLabel
                visible: false
                text: "there doesn't seem to be anything here"
                textStyle.color: Color.Red
                multiline: true
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                leftMargin: 20
                rightMargin: 20
            }
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onParentDataReply: {
                selfText = selfTextP;
                score = scoreP;
                author = authorP;
                likes = likesP;
                composeAction.enabled = true;
                permalink = permalinkP;
                console.log("permalink --" + permalinkP);
                if (pageTitle == "") {
                    pageTitle = titleP;
                }
            }
            onNextPageLoaded: {
                allowAtEndSignal = true;
            }
        },
        Sheet {
            id: commentReplySheet
            objectName: "composeMessageSheet"
            CommentReplySheet {
                id: commentReplySheetOut
                onBack: {
                    commentReplySheet.close();
                    commentReplySheet.deleteLater();
                }
            }
        },
        SystemToast {
            id: loginToastCompose
            body: "You must login to comment!"
        },
        PostActions {
            id: postActionsCompose
        },
        Sheet {
            id: editSheet
            EditSheet {
                id: editSheetOut
                onBack: {
                    editSheet.close();
                }
            }
        }
    ]
    onPostIDChanged: {
        feedHttp.getComments(postID, commentsIndicator, commentList, "confidence");
    }
    //onSelfTextChanged: {
    //leadingInfoLabel.setVisible(true);
    //leadingDivider.setVisible(true);
    //}
}
