import bb.cascades 1.0
import org.labsquare 1.0
import Network.FeedHttpCalls 1.0

Container {
    horizontalAlignment: HorizontalAlignment.Fill
    topMargin: 0
    signal commentBubbleClicked(variant data)
    id: feedItemContainer
    property bool showSubreddit
    property variant fontsize
    property bool isNSFW: ListItemData.data.over_18
    property string thumbnailUrl: ListItemData.data.thumbnail
    property string itemUrl: "https://www.reddit.com" + ListItemData.data.permalink
    property bool isRead: ListItemData.data.clicked
    property variant usersVote: getVote()
    property variant downVoteColor: Color.create("#6F72FB")
    property variant upVoteColor: Color.create("#F96E3B")
    property variant purpleColor: Color.create("#551A8B")
    function getVote() {
        if (ListItemData.data.likes == true) {
            return upVoteColor;
        } else if (ListItemData.data.likes == false) {
            return downVoteColor;
        } else {
            return Color.Black;
        }
    }
    Container {
        id: outer
        horizontalAlignment: HorizontalAlignment.Fill
        topPadding: 5
        bottomPadding: 5
        rightPadding: 5
        leftPadding: 5
        Container {
            id: inner
            horizontalAlignment: HorizontalAlignment.Fill
            topPadding: 10
            //rightPadding: 10
            leftPadding: 10
            bottomPadding: 10
            layout: StackLayout {
                orientation: LayoutOrientation.LeftToRight
            }
            Container {
                horizontalAlignment: HorizontalAlignment.Left
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                layoutProperties: StackLayoutProperties {
                    spaceQuota: 1
                }
                Container {
                    WebImageView {
                        id: thumbnailView
                        visible: setVisibility()
                        preferredHeight: 150
                        preferredWidth: 150
                        minWidth: 150
                        scalingMethod: ScalingMethod.AspectFit
                        url: thumbnailUrl
                        function setVisibility() {
                            if (thumbnailUrl.indexOf("http") != -1) {
                                return true;
                            } else {
                                return false;
                            }
                            /*if (thumbnailUrl) {
                                if (thumbnailUrl == "self" || thumbnailUrl == "" || thumbnailUrl == "default" || thumbnailUrl == "nsfw") {
                                    return false;
                                } else {
                                    return true;
                                }
                            } else {
                                return false;
                            }*/
                        }
                    }
                    touchPropagationMode: TouchPropagationMode.PassThrough
                }
                Container {
                    leftPadding: 10
                    //topPadding: 5.0
                    Label {
                        id: nsfwLabel
                        visible: isNSFW
                        textStyle.color: Color.Red
                        textStyle.fontSize: FontSize.Medium
                        text: "NSFW"
                        bottomMargin: 0
                    }
                    Label {
                        topMargin: 0
                        id: itemTitle
                        text: getTitle()
                        //content.flags: TextContentFlag.ActiveText
                        textStyle.fontSize: fontsize
                        textStyle.color: getClicked()
                        multiline: true
                        touchPropagationMode: TouchPropagationMode.PassThrough
                        function getTitle() {
                            if (ListItemData.data.title == null) {
                                return "<html>" + ListItemData.data.body + "</html>";
                            } else {
                                return "<html>" + ListItemData.data.title + "</html>";
                            }
                        }
                        function getClicked() {
                            var clicked = Qt.app.checkClicked(ListItemData.data.name);

                            if (clicked == true) {
                                return purpleColor;
                            } else {
                                return Color.Blue;
                            }
                        }
                        attachedObjects: [
                            FeedHttpCalls {
                                id: feedHttp
                            }
                        ]
                    }
                    Container {
                        id: subtextContainer
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        Label {
                            id: scoreLabel
                            text: ListItemData.data.score
                            textStyle.fontSize: FontSize.Small
                            textStyle.color: usersVote
                            rightMargin: 10
                        }
                        Label {
                            layoutProperties: StackLayoutProperties {
                                spaceQuota: 1
                            }
                            leftMargin: 5
                            text: getSubtitle()
                            textStyle.fontSize: FontSize.Small
                            textStyle.color: Color.DarkGray
                            function getSubtitle() {
                                if (showSubreddit) {
                                    return "by " + ListItemData.data.author + " to " + ListItemData.data.subreddit;
                                } else {
                                    return "by " + ListItemData.data.author;
                                }
                            }
                        }
                    }
                }
            }
            Container {
                rightPadding: 10
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Right
                Container {
                    horizontalAlignment: HorizontalAlignment.Center
                    bottomPadding: 5
                    Label {
                        id: timeLabel
                        text: milisecondsToString(ListItemData.data.created_utc)
                        textStyle.fontSize: FontSize.XSmall
                        textStyle.color: Color.DarkGray
                        textStyle.textAlign: TextAlign.Center

                        function milisecondsToString(miliseconds) {
                            var inputSeconds = parseInt(miliseconds);
                            var unixtime = inputSeconds * 1000;
                            var diff = new Date();
                            var unixtime2 = new Date(unixtime);

                            var secDiff = (diff.getSeconds() - unixtime2.getSeconds());
                            var minDiff = (diff.getMinutes() - unixtime2.getMinutes());
                            var hourDiff = (diff.getHours() - unixtime2.getHours());
                            var dayDiff = (diff.getDate() - unixtime2.getDate());
                            var monthDiff = (diff.getMonth() - unixtime2.getMonth());
                            var yearDiff = (diff.getFullYear() - unixtime2.getFullYear());
                            var months = [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ];
                            if (secDiff < 0) {
                                minDiff --;
                                secDiff += 60;
                            }
                            if (minDiff < 0) {
                                hourDiff --;
                                minDiff += 60;
                            }
                            if (hourDiff < 0) {
                                dayDiff --;
                                hourDiff += 24;
                            }
                            if (dayDiff < 0) {
                                monthDiff --;
                                dayDiff += months[diff.getMonth() - 2]; // -2 as the months array is zero-indexed too
                            }
                            if (monthDiff < 0) {
                                yearDiff --;
                                monthDiff += 12;
                            }
                            var digit = 0;
                            var timePeriod = "";

                            //check and return the largest value of date time initialized.
                            if (yearDiff > 0) {
                                digit = yearDiff;
                                timePeriod = "y";
                            } else if (monthDiff > 0) {
                                digit = monthDiff;
                                timePeriod = "M";
                            } else if (dayDiff != 0) {
                                digit = dayDiff;
                                timePeriod = "d";
                            } else if (hourDiff != 0) {
                                digit = hourDiff;
                                timePeriod = "h";
                            } else if (minDiff != 0) {
                                digit = minDiff;
                                timePeriod = "m";
                            } else if (secDiff != 0) {
                                digit = secDiff;
                                timePeriod = "s";
                            }
                            return digit + timePeriod + " ago";
                        }
                    }
                }
                Container {
                    layout: DockLayout {
                    }
                    minWidth: 100
                    ImageView {
                        imageSource: "images/commentBubble.png"
                        verticalAlignment: VerticalAlignment.Center
                        horizontalAlignment: HorizontalAlignment.Center
                        maxWidth: 150
                        scalingMethod: ScalingMethod.AspectFit
                    }
                    Container {
                        topPadding: 3
                        bottomPadding: 0
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Top
                        Label {
                            text: ListItemData.data.num_comments
                            //text: "500"
                            verticalAlignment: VerticalAlignment.Center
                            textStyle.color: Color.White
                            textStyle.fontSize: FontSize.PointValue
                            textStyle.fontSizeValue: 7
                            textStyle.textAlign: TextAlign.Center
                            horizontalAlignment: HorizontalAlignment.Center
                        }
                    }
                    onTouch: {
                        if (event.touchType == TouchType.Up) {
                            commentBubbleClicked(ListItemData);
                        }
                    }
                }
            }
        }
    }
    Divider {
        //preferredWidth:
        // horizontalAlignment: HorizontalAlignment.Fill
        topMargin: 0
        bottomMargin: 0
    }
    property variant blueBackground: Color.create("#00A7DD")
    gestureHandlers: [
        LongPressHandler {
            onLongPressed: {
                outer.background = blueBackground
                inner.background = blueBackground
            }
        }
    ]
    onTouch: {
        if (event.touchType == TouchType.Down) {
            outer.background = blueBackground
            inner.background = Color.White
        } else if (event.touchType == TouchType.Up || event.touchType == TouchType.Cancel) {
            inner.background = Color.Transparent;
            outer.background = Color.Transparent;
        }
    }
    onIsNSFWChanged: {
        //console.log("nsfw changed");
        if (! isNSFW) {
            nsfwLabel.visible = false;
        } else {
            var value = Qt.app.getSavedValue("hideNSFW", "");
            if (value == "true") {
                feedItemContainer.visible = false;
            } else {
                feedItemContainer.visible = true;
            }
            nsfwLabel.visible = isNSFW;
        }
    }
}
