import bb.cascades 1.2
import Network.FeedHttpCalls 1.0
import Network.PostActions 1.0
import bb.system 1.2

Page {
    property alias feedListShowSub: feedList.showSub
    property string subUrl
    property bool allowAtEndSignal
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    id: feedView
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            onTriggered: {
                //navPane.pop();
                feedView.parent.pop()
            }
            title: "Back"
        }
    }
    titleBar: TitleBar {
        property bool created: false
        id: titleBar
        title: qsTr("Feed")
        kind: TitleBarKind.Segmented
        options: [
            Option {
                text: "Hot"
                description: ""
                selected: true
            },
            Option {
                text: "New"
                description: "new/"
            },
            Option {
                id: topSegmentOption
                text: "Top"
                description: "top/"
                onTextChanged: {
                    console.log("text changed!");
                }
            }
        ]
        onSelectedIndexChanged: {
            if (selectedIndex == 2) {
                if(!created)
                {  
                    topFilterDialog.appendItem("past hour");
                    topFilterDialog.appendItem("past 24 hours");
                    topFilterDialog.appendItem("past week");
                    topFilterDialog.appendItem("past month", true, true);
                    topFilterDialog.appendItem("past year");
                    topFilterDialog.appendItem("all time");
                    created = true;
                }
                topFilterDialog.show();
            } else {
                feedHttp.getFeed(selectedOption.description, subUrl, activity, feedList);
            }
        }
        attachedObjects: [
            SystemListDialog {
                id: topFilterDialog
                title: "links from:"
                onFinished: {
                    switch(selectedIndices.toString()) {
                        case '0': {
                            feedHttp.getFeed("top/?sort=top&t=hour&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: Hour");
                            break;
                            }
                        case '1': {
                            feedHttp.getFeed("top/?sort=top&t=day&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: Day");
                            break;
                            }
                        case '2': {
                            feedHttp.getFeed("top/?sort=top&t=week&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: Week");
                            break;
                            }
                        case '3': {
                            feedHttp.getFeed("top/?sort=top&t=month&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: Month");
                            break;
                            }    
                        case '4': {
                            feedHttp.getFeed("top/?sort=top&t=year&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: Year");
                            break;
                            }
                        case '5': {
                            feedHttp.getFeed("top/?sort=top&t=all&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: All");
                            break;
                            }
                        default: {
                            feedHttp.getFeed("top/?sort=top&t=all&dumb=", subUrl, activity, feedList);
                            topSegmentOption.setText("Top: All");
                        }
                    }
                    //selectedIndices
                }        
            }
        ]
    }

    actions: [
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("p")
                }
            ]
            title: "Submit Post"
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/composeMessageIcon.png"
            onTriggered: {
                if (Qt.app.getLoggedIn() == false) {
                    loginToastPost.show();
                } else {
                    composeThreadSheet.open();
                }
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("i")
                }
            ]
            title: "Info"
            enabled: (feedView.subUrl != '/') 
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/ic_info.png"
            onTriggered: {
                var page = infoView.createObject();
                page.subUrl = feedView.subUrl;
                feedView.parent.push(page);
            }
            attachedObjects: [
                ComponentDefinition {
                    id: infoView
                    source: "InfoView.qml"
                }
            ]
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToTop
                }
            ]
            title: "To Top"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_top.png"
            onTriggered: {
                feedList.scrollToItem([ 0 ]);
            }
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToBottom
                }
            ]
            title: "To Bottom"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_bottom.png"
            onTriggered: {
                feedList.scrollToItem([ feedList.dataModel.size() - 1 ]);
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("r")
                }
            ]
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: "Refresh"
            imageSource: "asset:///images/reloadIcon.png"
            onTriggered: {
                feedView.refresh();
                //feedHttp.getFeed(titleBar.selectedOption.description, subUrl, activity, feedList);
            }
        }
    ]
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        layout: DockLayout {
            
        }
        ListView {
            property bool loggedIn: checkLoggedIn()
            property variant fontSize: getFontsize()
            property string subreddit
            property bool showSub
            scrollRole: ScrollRole.Main
            objectName: "feedList"
            id: feedList
            layout: StackListLayout {
                headerMode: ListHeaderMode.Standard
            }
            listItemComponents: [
                ListItemComponent {
                    type: "header"
                    Header {
                        id: headerItem
                        title: headerItem.ListItem.view.subreddit
                        subtitle: "Page " + ListItemData.data.page
                    }
                },
                ListItemComponent {
                    id: listItemRoot
                    type: "item"
                    FeedListItem {
                        id: feedListItem
                        fontsize: feedListItem.ListItem.view.fontSize
                        showSubreddit: feedListItem.ListItem.view.showSub
                        contextActions: [
                            ActionSet {
                                id: contextActionSet
                                title: ListItemData.data.title
                                subtitle: ListItemData.data.author
                                ActionItem {
                                    shortcuts: [
                                        Shortcut {
                                            key: qsTr("u")
                                        }
                                    ]
                                    id: upvoteAction
                                    title: "Upvote"
                                    imageSource: "asset:///images/upvoteIcon.png"
                                    onTriggered: {
                                        if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                            loginToast.body = "You must login to vote!";
                                            loginToast.show();
                                        } else {
                                            var selectedItem = ListItemData;
                                            /*if (selectedItem.data.likes == "undefined") {
                                             * return;
                                             * }*/
                                            if (selectedItem.data.likes == true) {
                                                return;
                                                /*postActions.votePost(selectedItem.data.name, "0");
                                                 * selectedItem.data.likes = "undefined";
                                                 * selectedItem.data.score += 1;
                                                 * feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);*/
                                            }
                                            if (selectedItem.data.likes == false) {
                                                postActions.votePost(selectedItem.data.name, "1");
                                                selectedItem.data.likes = true;
                                                selectedItem.data.score += 2;
                                                feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                            } else {
                                                postActions.votePost(selectedItem.data.name, "1");
                                                selectedItem.data.likes = true;
                                                selectedItem.data.score += 1;
                                                feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                            }
                                        }
                                    }
                                }
                                ActionItem {
                                    shortcuts: [
                                        Shortcut {
                                            key: qsTr("d")
                                        }
                                    ]
                                    id: downvoteAction
                                    title: "Downvote"
                                    imageSource: "asset:///images/downvoteIcon.png"
                                    onTriggered: {
                                        if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                            loginToast.body = "You must login to vote!";
                                            loginToast.show();
                                        } else {
                                            var selectedItem = ListItemData;
                                            /*if (selectedItem.data.likes == "undefined") {
                                             * return;
                                             * }*/
                                            if (selectedItem.data.likes == false) {
                                                return;
                                                /*postActions.votePost(selectedItem.data.name, "0");
                                                 * selectedItem.data.likes = "undefined";
                                                 * selectedItem.data.score += 1;
                                                 * feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);*/
                                            }
                                            if (selectedItem.data.likes == true) {
                                                postActions.votePost(selectedItem.data.name, "-1");
                                                selectedItem.data.likes = false;
                                                selectedItem.data.score -= 2;
                                                feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                            } else {
                                                postActions.votePost(selectedItem.data.name, "-1");
                                                selectedItem.data.likes = false;
                                                selectedItem.data.score -= 1;
                                                feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                            }
                                        }
                                    }
                                }
                                ActionItem {
                                    shortcuts: [
                                        Shortcut {
                                            key: qsTr("c")
                                        }
                                    ]
                                    title: "Comments (" + ListItemData.data.num_comments + ")"
                                    imageSource: "asset:///images/commentsIcon.png"
                                    onTriggered: {
                                        var selectedItem = ListItemData;
                                        var page = feedDetailsViewList.createObject();
                                        page.pageTitle = selectedItem.data.title;
                                        page.postID = (selectedItem.data.name);
                                        feedListItem.ListItem.view.pushPageStack(page);
                                    }
                                }
                                ActionItem {
                                    shortcuts: [
                                        Shortcut {
                                            key: qsTr("s")
                                        }
                                    ]
                                    id: saveAction
                                    title: "Save"
                                    imageSource: "asset:///images/star.png"
                                    onTriggered: {
                                        if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                            loginToast.body = "You must login to save posts!";
                                            loginToast.show();
                                        } else {
                                            var selectedItem = ListItemData;
                                            if (selectedItem.data.saved == false) {
                                                postActions.savePost(selectedItem.data.name, "save");
                                                selectedItem.data.saved = true;
                                                feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                            } else {
                                                postActions.savePost(selectedItem.data.name, "unsave");
                                                selectedItem.data.saved = false;
                                                feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                            }
                                        }
                                    }
                                }
                                ActionItem {
                                    shortcuts: [
                                        Shortcut {
                                            key: qsTr("h")
                                        }
                                    ]
                                    id: hideAction
                                    title: "Hide"
                                    imageSource: "asset:///images/hide.png"
                                    onTriggered: {
                                        if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                            loginToast.body = "You must login to hide posts!";
                                            loginToast.show();
                                        } else {
                                            var selectedItem = ListItemData;
                                            if (selectedItem.data.hidden == false) {
                                                postActions.hidePost(true, selectedItem.data.name);
                                                feedListItem.ListItem.view.dataModel.removeAt(feedListItem.ListItem.indexPath);
                                            } else {
                                                postActions.hidePost(false, selectedItem.data.name);
                                                feedListItem.ListItem.view.dataModel.removeAt(feedListItem.ListItem.indexPath);
                                            }
                                        }
                                    }
                                }
                                InvokeActionItem {
                                    id: shareAction
                                    objectName: "shareAction"
                                    title: "Share"
                                    query {
                                        mimeType: "text/plain"
                                        invokeActionId: "bb.action.SHARE"
                                    }
                                    onTriggered: {
                                        data = ListItemData.data.title + " - https://www.reddit.com" + ListItemData.data.permalink + " via Reddit in Motion";
                                    }
                                }
                                ActionItem {
                                    shortcuts: [
                                        Shortcut {
                                            key: qsTr("g")
                                        }
                                    ]
                                    id: gotoAction
                                    imageSource: "asset:///images/gotoIcon.png"
                                    objectName: "gotoAction"
                                    title: "Goto"
                                    //enabled: feedListShowSub
                                    onTriggered: {
                                        var selectedItem = ListItemData;
                                        var page = feedViewGoto.createObject();
                                        var subUrl = selectedItem.data.subreddit;
                                        var firstSlash = subUrl.lastIndexOf("/");
                                        if (firstSlash == -1) {
                                            page.subUrl = "/r/" + subUrl + "/";
                                        } else {
                                            page.subUrl = "/r" + subUrl.substring(firstSlash) + "/";
                                        }
                                        feedListItem.ListItem.view.pushPageStack(page);
                                    }
                                }
                                attachedObjects: [
                                    SystemToast {
                                        id: loginToast
                                        body: "You must login to vote!"
                                    },
                                    ComponentDefinition {
                                        id: feedViewGoto
                                        source: "FeedView.qml"
                                    }
                                ]
                                onCreationCompleted: {
                                    if (! feedListItem.ListItem.view.getShowSub()) {
                                        contextActionSet.remove(gotoAction);
                                    }
                                    //addAction(gotoAction);
                                }
                            }
                        ]
                        attachedObjects: [
                            ComponentDefinition {
                                id: feedDetailsViewList
                                source: "FeedDetails.qml"
                            },
                            PostActions {
                                id: postActions
                            }
                        ]
                        contextMenuHandler: ContextMenuHandler {
                            id: contextHandler
                            onPopulating: {
                                if (ListItemData.data.likes == true) {
                                    upvoteAction.setImageSource("asset:///images/upvoteSelectedIcon.png");
                                    downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                } else if (ListItemData.data.likes == false) {
                                    downvoteAction.setImageSource("asset:///images/downvoteSelectedIcon.png");
                                    upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                } else {
                                    upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                    downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                }
                                if (ListItemData.data.saved == true) {
                                    saveAction.setImageSource("asset:///images/saved.png");
                                    saveAction.setTitle("Unsave");
                                } else {
                                    saveAction.setImageSource("asset:///images/star.png");
                                    saveAction.setTitle("Save");
                                }
                                if (ListItemData.data.hidden == true) {
                                    hideAction.setImageSource("asset:///images/unhide.png");
                                    hideAction.setTitle("Unhide");
                                } else {
                                    hideAction.setImageSource("asset:///images/hide.png");
                                    hideAction.setTitle("Hide");
                                }
                                if (gotoAction) {
                                    gotoAction.setTitle("Goto (r/" + ListItemData.data.subreddit + ")");
                                }
                            }
                        }
                        onCommentBubbleClicked: {
                            var selectedItem = data;
                            var page = feedDetailsViewList.createObject();
                            page.pageTitle = selectedItem.data.title;
                            page.postID = (selectedItem.data.name);
                            feedListItem.ListItem.view.pushPageStack(page);
                        }
                    }
                },
                ListItemComponent {
                    id: footerListItem
                    type: "footer"
                    Container {
                        touchPropagationMode: TouchPropagationMode.None
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        id: footerContainer
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        ActivityIndicator {
                            running: true
                            preferredWidth: 100
                            preferredHeight: 100
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                        }
                        Label {
                            verticalAlignment: VerticalAlignment.Center
                            text: "loading more"
                        }
                    }
                }
            ] //ListItemComponents
            // Treat outer level as header, inner level as item
            function itemType(data, indexPath) {
                var currentItem = dataModel.data(indexPath);
                if (currentItem.data.type == "header") {
                    var lastSlash = subUrl.lastIndexOf("/");
                    subreddit = subUrl.substring(3, lastSlash);
                    if (subreddit == "/") { //Otherwise, frontpage will show up as "/"
                        if (app.getLoggedIn()) {
                            subreddit = "frontpage"
                        } else {
                            subreddit = "frontpage (default)"    
                        }
                    }
                    return "header";
                } else if (currentItem.data.type == "footer") {
                    return "footer";
                } else {
                    return "item"
                }
            }
            function htmlDecode(html) {
                html = html.replace(new RegExp("&amp;", "g"), String.fromCharCode(38));
                return html;
            }
            onTriggered: {
                var page;
                var selectedItem = dataModel.data(indexPath);

                if (selectedItem.kind == "t1") { //probably unnecessary since no t1 posts in feeds
                    var page = feedDetailsView.createObject();
                    page.pageTitle = selectedItem.data.link_title;
                    page.postID = selectedItem.data.link_id;
                } else {
                    var domain = selectedItem.data.domain;
                    var found = domain.search('self');
                    if (found != -1) { //if a 'self' post
                        page = feedDetailsView.createObject();
                        page.pageTitle = selectedItem.data.title;
                        page.postID = selectedItem.data.name;
                    } else {
                        var getOpenLinksBrowser = Qt.app.getSavedValue("openLinksBrowser", "");
                        if (getOpenLinksBrowser == "true") {
                            Qt.app.invokeBrowser(selectedItem.data.url);
                        } else {
                            page = Qt.webView.createObject();
                            console.log("URI -- " + selectedItem.data.url);
                            console.log("URI encodedComponent -- " + htmlDecode(selectedItem.data.url));
                            page.webUrl = htmlDecode(selectedItem.data.url);
                            page.pageTitle = selectedItem.data.title;
                            page.postID = selectedItem.data.name;
                        }
                    }
                    Qt.app.addClicked(selectedItem.data.name); //wouldn't we want to also mark a t1 post as read too? not sure if the name/link_id thing is different
                }
                selectedItem.data.clicked = true;
                dataModel.replace(indexPath, selectedItem);
                feedView.parent.push(page)
            }
            onDataModelChanged: {
                fontSize = getFontsize();
                if (dataModel.size() <= 2) {
                    allowAtEndSignal = false;
                    emptyLabel.setVisible(true);
                } else {
                    allowAtEndSignal = true;
                    emptyLabel.setVisible(false);
                }
            }

            attachedObjects: [
                /*ComponentDefinition {
                 * id: webView
                 * source: "PostWebView.qml"
                 * },*/
                ComponentDefinition {
                    id: feedDetailsView
                    source: "FeedDetails.qml"
                },
                ListScrollStateHandler {
                    onAtEndChanged: {
                        if (atEnd == true && allowAtEndSignal == true) {
                            allowAtEndSignal = false;
                            feedHttp.getNextPage(titleBar.selectedOption.description, subUrl, feedList);
                        }
                    }
                }
            ]
            function pushPageStack(pageParam) {
                //navPane.push(pageParam);
                feedView.parent.push(pageParam);
            }
            function checkLoggedIn() {
                if (Qt.app)
                    return Qt.app.getLoggedIn();
                else return app.getLoggedIn();
            }
            function getShowSub() {
                return showSub;
            }
            function getFontsize() {
                var value;
                if (Qt.app) {
                    value = Qt.app.getSavedValue("fontsize", "");
                } else {
                    value = app.getSavedValue("fontsize", "");
                }
                if (value == "Large") {
                    return FontSize.Large;
                } else if (value == "Medium") {
                    return FontSize.Medium;
                } else return FontSize.Small;
            }
        }
        ActivityIndicator {
            objectName: "indicator"
            id: activity
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
        Label {
            id: emptyLabel
            visible: false
            text: "there doesn't seem to be anything here"
            textStyle.color: Color.Red
            multiline: true
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            leftMargin: 20
            rightMargin: 20
        }
    }
    attachedObjects: [
        FeedHttpCalls {
            id: feedHttp
            onNextPageLoaded: {
                allowAtEndSignal = true;
            }
        },
        Sheet {
            id: composeThreadSheet
            ComposeThreadSheet {
                currentSub: getSub()
                onBack: {
                    composeThreadSheet.close();
                }
                onPostThread: {
                    composeThreadSheet.close();
                }
                function getSub() {
                    if (feedList.subreddit == "frontpage" || feedList.subreddit == "/") {
                        return "";
                    } else {
                        return feedList.subreddit;
                    }
                }
            }
        },
        SystemToast {
            id: loginToastPost
            body: "You must login to submit posts!"
        }
    ]
    onSubUrlChanged: {
        /*if (subUrl == "/r/all/" || subUrl == "/") {
         * feedListShowSub = true;
         * }*/
        feedHttp.getFeed("", subUrl, activity, feedList);
    }
    function refresh() {
        feedHttp.getFeed(titleBar.selectedOption.description, subUrl, activity, feedList);
    }
}
