import bb.cascades 1.2
import org.labsquare 1.0
import "markdown2html.js" as MarkdownConverter 

Container {
    horizontalAlignment: HorizontalAlignment.Fill
    topMargin: 0
    id: feedItemContainer
    property bool isNew: ListItemData.data["new"]
    property string itemUrl: "https://www.reddit.com" + ListItemData.data.permalink
    property alias subjectLabel: subjectLabel
    Container {
        id: outer
        horizontalAlignment: HorizontalAlignment.Fill
        topPadding: 5
        bottomPadding: 5
        rightPadding: 5
        leftPadding: 5
        Container {
            id: inner
            horizontalAlignment: HorizontalAlignment.Fill
            topPadding: 10
            rightPadding: 10
            leftPadding: 10
            bottomPadding: 10
            Container {
                id: subjectContainer
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                Label {
                    id: subjectLabel
                    text: ListItemData.data.subject
                    textStyle.fontWeight: FontWeight.Bold
                    textStyle.color: checkIfNew()
                    function checkIfNew() {
                        if (isNew == true) {
                            return Color.create("#FF4500");
                        } else {
                            return Color.Black
                        }
                    }
                }
                Label {
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1
                    }
                    id: linkTitle
                    text: getLinktitle()
                    textStyle.fontStyle: FontStyle.Italic
                    function getLinktitle() {
                        if (ListItemData.data.link_title == undefined) {
                            return ""
                        } else {
                            return ListItemData.data.link_title;
                        }
                    }
                }
            }
            Label {
                id: itemTitle
                text: getTitle()
                content.flags: TextContentFlag.ActiveText
                textStyle.fontSize: FontSize.Medium
                multiline: true
                function getTitle() {
                    var titleValue;
                    if (ListItemData.data.title == null) {
                        titleValue = MarkdownConverter._RunSpanGamut(ListItemData.data.body);
                        //titleValue = "<html>" + titleValue + "</html>"
                        //titleValue = ListItemData.data.body;
                    } else {
                        titleValue = ListItemData.data.title;
                    }
                    if (titleValue.length > 150) {
                        titleValue = titleValue.substring(0, 150) + "...[read more]";
                    }
                    return "<html>" + titleValue + "</html>";
                }
            }
            Container {
                layout: StackLayout {
                    orientation: LayoutOrientation.LeftToRight
                }
                Label {
                    text: getFromText()
                    textStyle.fontSize: FontSize.Medium
                    textStyle.color: Color.DarkGray
                    function getFromText() {
                        var subString;
                        var toOrFrom;
                        if (Qt.messagesType == "inbox") {
                            toOrFrom = "from "
                        } else {
                            toOrFrom = "to "
                        }
                        if (ListItemData.kind == "t1") {
                            subString = "from " + ListItemData.data.author + " via " + ListItemData.data.subreddit;
                        } else if (ListItemData.kind == "t4") {
                            subString = toOrFrom + ListItemData.data.author;
                        }
                        return subString;
                    }
                }
                Label {
                    id: timeLabel
                    text: milisecondsToString(ListItemData.data.created_utc)
                    textStyle.fontSize: FontSize.Medium
                    textStyle.color: Color.DarkGray
                    textStyle.textAlign: TextAlign.Center

                    function milisecondsToString(miliseconds) {
                        var inputSeconds = parseInt(miliseconds);
                        var unixtime = inputSeconds*1000;
                        var diff = new Date();
                        var unixtime2 = new Date(unixtime);
                        
                        var secDiff = (diff.getSeconds() - unixtime2.getSeconds());
                        var minDiff = (diff.getMinutes() - unixtime2.getMinutes());
                        var hourDiff = (diff.getHours() - unixtime2.getHours());
                        var dayDiff = (diff.getDate() - unixtime2.getDate());
                        var monthDiff = (diff.getMonth() - unixtime2.getMonth());
                        var yearDiff = (diff.getFullYear() - unixtime2.getFullYear());
                        var months = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
                        if (secDiff < 0) {
                            minDiff--;
                            secDiff += 60;
                        }
                        if (minDiff < 0) {
                            hourDiff--;
                            minDiff += 60;
                        }
                        if (hourDiff < 0) {
                            dayDiff--;
                            hourDiff += 24;
                        }
                        if (dayDiff < 0) {
                            monthDiff--;
                            dayDiff += months[diff.getMonth()-2]; // -2 as the months array is zero-indexed too
                        }
                        if (monthDiff < 0) {
                            yearDiff--;
                            monthDiff += 12;
                        }
                        var digit = 0;
                        var timePeriod = "";
                        
                        //check and return the largest value of date time initialized.
                        if (yearDiff > 0) {
                            digit=yearDiff;
                            timePeriod = "y";
                        }
                        else if (monthDiff > 0) {
                            digit=monthDiff;
                            timePeriod="M";
                        }
                        else if (dayDiff != 0) {
                            digit=dayDiff;
                            timePeriod="d";
                        } else if (hourDiff != 0) {
                            digit=hourDiff;
                            timePeriod="h";
                        } else if (minDiff != 0) {
                            digit=minDiff;
                            timePeriod="m";
                        } else if (secDiff != 0) {
                            digit=secDiff;
                            timePeriod="s";
                        }
                        return digit + timePeriod + " ago";
                    }
                }
            }
        }
    }
    Divider {
        //preferredWidth: 768
        topMargin: 0
        bottomMargin: 0
    }
    property variant blueBackground: Color.create("#00A7DD")
    gestureHandlers: [
        LongPressHandler {
            onLongPressed: {
                outer.background = blueBackground
                inner.background = blueBackground
            }
        }
    ]
    onTouch: {
        if (event.touchType == TouchType.Down) {
            outer.background = blueBackground
            inner.background = Color.White
        } else if (event.touchType == TouchType.Up || event.touchType == TouchType.Cancel) {
            inner.background = Color.Transparent;
            outer.background = Color.Transparent;
        }
    }
}
