import bb.cascades 1.2
import Network.MessagesHttp 1.0

Page {
    id: messagesPage
    property bool allowAtEndSignal
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    actions: [
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("c")
                }
            ]
            title: "Compose"
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/composeMessageIcon.png"
            onTriggered: {
                composeMessageSheet.open();
                //messagesHttp.requestCaptcha(); //doesnt seem like it actually needs real captcha values
                messagesHttp.checkNeedCaptcha();
            }
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToTop
                }
            ]
            title: "To Top"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_top.png"
            onTriggered: {
                messagesList.scrollToItem([ 0 ]);
            }
        },
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.JumpToBottom
                }
            ]
            title: "To Bottom"
            ActionBar.placement: ActionBarPlacement.InOverflow
            imageSource: "asset:///images/ic_to_bottom.png"
            onTriggered: {
                messagesList.scrollToItem([ messagesList.dataModel.size() - 1 ]);
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("r")
                }
            ]
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: "Refresh"
            imageSource: "asset:///images/reloadIcon.png"
            onTriggered: {
                messagesHttp.getMessages(messagesDropDown.selectedOption.text, messagesActivity, messagesList);
                messagesHttp.getUnreadCount(tab3);
            }
        }
    ]
    titleBar: TitleBar {
        id: titlebar3
        title: "messages"
        kind: TitleBarKind.FreeForm
        kindProperties: FreeFormTitleBarKindProperties {
            content: Container {
                leftPadding: 15
                topPadding: 25
                background: Color.create("#0C87BF")
                Label {
                    text: "messages"
                    textStyle.fontSize: FontSize.Large
                    textStyle.color: Color.White
                }
            }
        }
    }
    content: Container {
        layout: DockLayout {
        }
        ListView {
            scrollRole: ScrollRole.Main
            id: messagesList
            leadingVisual: Container {
                horizontalAlignment: HorizontalAlignment.Center
                DropDown {
                    title: "sorting:"
                    id: messagesDropDown
                    horizontalAlignment: HorizontalAlignment.Center
                    preferredWidth: Infinity
                    bottomMargin: 0
                    options: [
                        Option {
                            text: "inbox"
                            selected: true
                        },
                        Option {
                            text: "sent"
                        }
                    ]
                    onSelectedIndexChanged: {
                        Qt.messagesType = selectedOption.text;
                        messagesHttp.getMessages(selectedOption.text, messagesActivity, messagesList);
                    }
                }
                Divider {
                    topMargin: 0
                    bottomMargin: 0
                }
            }
            listItemComponents: [
                ListItemComponent {
                    type: "item"
                    MessageListItem {
                        id: messagesListItem
                        contextActions: ActionSet {
                            title: ListItemData.data.subject
                            subtitle: getsubTitle()
                            ActionItem {
                                id: replyAction
                                title: "Reply"
                                imageSource: "asset:///images/ic_reply_as_email.png"
                                onTriggered: {
                                    var selectedItem = ListItemData;
                                    messagesListItem.ListItem.view.pushReplyComment(selectedItem);
                                }
                            }
                            ActionItem {
                                id: markUnreadAction
                                title: "Mark Unread"
                                imageSource: "asset:///images/mark_unread.png"
                                onTriggered: {
                                    if (messagesListItem.isNew == false) {
                                        var selectedItem = ListItemData;
                                        selectedItem.data["new"] = true;
                                        messagesListItem.ListItem.view.dataModel.replace(messagesListItem.ListItem.indexPath, selectedItem);
                                        //ListItemData.data["new"] = true;
                                        //messagesListItem.isNew = true;
                                        Qt.tab3.unreadContentCount += 1;
                                        messagesListItem.ListItem.view.markMessageUnread(selectedItem.data.name);
                                    }
                                }
                            }
                            ActionItem {
                                id: viewFullThread
                                title: "View Full Thread"
                                imageSource: "asset:///images/commentsIcon.png"
                                enabled: checkFullThread()
                                onTriggered: {
                                    messagesListItem.ListItem.view.openFullThread(ListItemData, messagesListItem.ListItem.indexPath);
                                }
                                function checkFullThread() {
                                    if (ListItemData.kind == "t1") {
                                        return true;
                                    } else return false;
                                }
                            }
                            function getsubTitle() {
                                var text;
                                if (ListItemData.data.title == null) {
                                    text = ListItemData.data.body;
                                } else {
                                    text = ListItemData.data.title;
                                }
                                return removeNewLines(text);
                            }
                            function removeNewLines(text) {
                                var noLines = text;
                                var intIndexOfMatch = noLines.indexOf("\n");
                                while (intIndexOfMatch != -1) {
                                    noLines = noLines.replace("\n", " ");
                                    intIndexOfMatch = noLines.indexOf("\n");
                                }
                                return noLines;
                            }
                        }
                        contextMenuHandler: ContextMenuHandler {
                            id: messageContextHandler
                            onPopulating: {
                                var isSent = messagesListItem.ListItem.view.checkType();
                                if (isSent) {
                                    markUnreadAction.setEnabled(false);
                                    replyAction.setEnabled(false)
                                    //event.abort();
                                } else {
                                    markUnreadAction.setEnabled(true);
                                    replyAction.setEnabled(true)
                                }
                            }
                        }
                    
                    }
                },
                ListItemComponent {
                    id: footerListItem
                    type: "footer"
                    Container {
                        touchPropagationMode: TouchPropagationMode.None
                        layout: StackLayout {
                            orientation: LayoutOrientation.LeftToRight
                        }
                        id: footerContainer
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        ActivityIndicator {
                            running: true
                            preferredWidth: 100
                            preferredHeight: 100
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                        }
                        Label {
                            verticalAlignment: VerticalAlignment.Center
                            text: "loading more"
                        }
                    }
                }
            ] //ListItemComponents
            function itemType(data, indexPath) {
                var currentItem = dataModel.data(indexPath);
                if (currentItem.data.type == "footer") {
                    return "footer";
                } else {
                    return "item"
                }
            }
            function checkType() {
                if (messagesDropDown.selectedOption.text == "sent") {
                    return true;
                } else return false;
            }
            function pushReplyComment(selectedItem) {
                var page = messageDetailsView.createObject();
                page.subject = selectedItem.data.subject;
                page.dataObject = selectedItem;
                messagesNavPane.push(page);
            }
            function markMessageUnread(name) {
                messagesHttp.markRead(name, "unread_message");
            }
            function openFullThread(selectedItem, indexP) {
                var page = feedDetailsView.createObject();
                page.pageTitle = selectedItem.data["link_title"];
                var context = selectedItem.data["context"];
                var begin = context.indexOf("/comments/") + 10;
                var end = context.indexOf("/", begin);
                page.postID = context.substring(0, end+1);
                messagesNavPane.push(page);
                var isNew = selectedItem.data;
                if (isNew["new"] == true) {
                    selectedItem.data["new"] = false;
                    messagesHttp.markRead(selectedItem.data.name, "read_message");
                    dataModel.replace(indexP, selectedItem);
                    tab3.unreadContentCount -= 1;
                }
            }
            onTriggered: {
                var selectedItem = dataModel.data(indexPath);
                if (selectedItem.kind == "t1") { //comment or post reply is t1, plain message is t4
                    var page = feedDetailsView.createObject();
                    page.pageTitle = selectedItem.data.link_title;
                    page.postID = selectedItem.data.context;
                } else {
                    var page = messageDetailsView.createObject();
                    page.subject = selectedItem.data.subject;
                    page.dataObject = selectedItem;
                }
                messagesNavPane.push(page);
                var isNew = selectedItem.data;
                if (isNew["new"] == true) {
                    selectedItem.data["new"] = false;
                    messagesHttp.markRead(selectedItem.data.name, "read_message");
                    dataModel.replace(indexPath, selectedItem);
                    tab3.unreadContentCount -= 1;
                }
            }
            onCreationCompleted: {
                messagesHttp.getMessages("inbox", messagesActivity, messagesList);
                //messagesHttp.getUnreadCount(tab3);
            }
            onDataModelChanged: {
                if (dataModel.size() <= 0) {
                    messagesPage.allowAtEndSignal = false;
                    emptyLabelMessages.setVisible(true);
                } else {
                    messagesPage.allowAtEndSignal = true;
                    emptyLabelMessages.setVisible(false);
                }
            }
            attachedObjects: [
                ListScrollStateHandler {
                    onAtEndChanged: {
                        if (atEnd == true && messagesPage.allowAtEndSignal == true) {
                            messagesPage.allowAtEndSignal = false;
                            messagesHttp.getNextPage(messagesDropDown.selectedOption.text, messagesList);
                        }
                    }
                }
            ]
        }
        ActivityIndicator {
            id: messagesActivity
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
        Label {
            id: emptyLabelMessages
            visible: false
            text: "there doesn't seem to be anything here"
            textStyle.color: Color.Red
            multiline: true
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            leftMargin: 20
            rightMargin: 20
        }
    }
    attachedObjects: [
        MessagesHttp {
            id: messagesHttp
            onNextPageLoaded: {
                messagesPage.allowAtEndSignal = true;
            }
        },
        ComponentDefinition {
            id: messageDetailsView
            source: "MessageDetailsView.qml"
        },
        ComponentDefinition {
            id: feedDetailsView
            source: "FeedDetails.qml"
        },
        Sheet {
            id: composeMessageSheet
            objectName: "composeMessageSheet"
            ComposeMessageSheet {
                id: composeMessageSheetOutside
                onBack: {
                    composeMessageSheet.close();
                    composeMessageSheet.deleteLater();
                }
                onSendMessage: {
                    composeMessageSheet.close();
                    composeMessageSheet.deleteLater();
                }
            }
        }
    ]
}