import bb.cascades 1.2
import bb.system 1.2
import bb.cascades.pickers 1.0
import org.labsquare 1.0

Page {
    property string webUrl
    property string pageTitle
    property string postID
    property alias webView: webView
    property string readerUrl: "https://www.readability.com/m?url="
    property string desktopUserAgentString: "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_5) AppleWebKit/600.1.17 (KHTML, like Gecko) Version/7.1 Safari/537.85.10"
    property string mobileUserAgentString: "Mozilla/5.0 (BB10; Kbd) AppleWebKit/537.35+ (KHTML, like Gecko) Version/10.3.2.2639 Mobile Safari/537.35+"
    
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    titleBar: TitleBar {
        title: pageTitle
    }
    id: webViewPage
    objectName: "webViewPage"
    actions: [
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("b")
                }
            ]
            title: "Browser Back"
            imageSource: "asset:///images/ic_previous.png"
            enabled: (webView.canGoBack) ? true : false
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                webView.goBack();
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("c")
                }
            ]
            id: commentsAction
            enabled: false
            title: "Comments"
            imageSource: "asset:///images/commentsIcon.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                var page = feedDetailsView.createObject();
                page.pageTitle = pageTitle;
                page.postID = postID;
                webViewPage.parent.peekEnabled = false;
                webViewPage.parent.push(page);
            }
        },
        /*ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("r")
                }
            ]
            id: readerModeAction
            title: "Reader Mode"
            imageSource: "asset:///images/ic_view_details.png"
            onTriggered: {
                webView.url = readerUrl + webUrl;
            }
        },*/
        InvokeActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("o")
                }
            ]
            id: invokeB
            title: "Open in Browser"
            //imageSource: "images/globe.png"
            query {
                uri: webUrl
                mimeType: "text/html"
                invokeActionId: "bb.action.OPEN"
                onQueryChanged: invokeB.query.updateQuery()
            }

        },
        ActionItem {
            title: "Copy Link"
            imageSource: "asset:///images/ic_copy_link.png"
            onTriggered: {
                Qt.app.copyToClipboard(webUrl);
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("l")
                }
            ]
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: "Refresh"
            imageSource: "asset:///images/reloadIcon.png"
            onTriggered: {
                webView.reload();
            }
        },
        InvokeActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("s")
                }
            ]
            id: shareAction
            objectName: "shareAction"
            title: "Share"
            query {
                mimeType: "text/plain"
                invokeActionId: "bb.action.SHARE"
            }
            onTriggered: {
                data = pageTitle + " - " + webUrl + " via Reddit in Motion";
            }
        }
    ]
    attachedObjects: [
        ComponentDefinition {
            id: feedDetailsView
            source: "FeedDetails.qml"
        }
    ]
    function onInstallCertsFinished() {
        webView.url = webView.lastUrlRequested
    }
    content: Container {
        //horizontalAlignment: HorizontalAlignment.Fill
        //verticalAlignment: VerticalAlignment.Fill
        visible: true
        id: imageViewContainer
        layout: DockLayout {
        }
        ScrollView {
            id: scrollView
            scrollRole: ScrollRole.Main
            scrollViewProperties {
                scrollMode: ScrollMode.Both
                pinchToZoomEnabled: true
                initialScalingMethod: ScalingMethod.AspectFit
                maxContentScale: 2.5
                minContentScale: 1.0
            }
            //verticalAlignment: VerticalAlignment.Fill
            //horizontalAlignment: HorizontalAlignment.Fill
            /*onContentScaleChanged: {
                if (contentScale > 1.0) {
                    scrollViewProperties.scrollMode = ScrollMode.Both;
                } else {
                    scrollViewProperties.scrollMode = ScrollMode.Vertical;
                }
            }*/
            WebView {
                preferredHeight: Infinity
                preferredWidth: Infinity
                id: webView
                objectName: "webView"
                settings.zoomToFitEnabled: true
                settings.credentialAutoFillEnabled: true
                //settings.userAgent: desktopUserAgentString
                property string lastUrlRequested
                onLoadProgressChanged: {
                    progress.setValue(loadProgress);
                }
                onLoadingChanged: {
                    if (loading) {
                        progress.setVisible(true);
                    } else {
                        progress.setVisible(false);
                    }
                    if (loadRequest.status == WebLoadStatus.Failed) {
                        console.log("web load failed!!");
                        if (lastUrlRequested.indexOf("gfycat.com") > -1) {
                            installCertsDialog.show()
                        }
                    }
                }
                /*onUrlChanged: {
                    var urlString = url.toString();
                    if (urlString.indexOf("readability") > -1) {
                        readerModeAction.setEnabled(false);
                    } else {
                        readerModeAction.setEnabled(true);
                    }
                }*/
                onNavigationRequested: {
                    //app.addCustomCerts();
                    var purl = request.url.toString();
                    console.log(purl);
                    /*if (purl.indexOf("imgur.com")!= -1) {
                        webView.settings.setUserAgent(desktopUserAgentString);
                    } else {
                        webView.settings.setUserAgent(mobileUserAgentString);
                    }*/
                    lastUrlRequested = purl;
                    
                }
                attachedObjects: [
                    FilePicker {
                        id:filePicker
                        type: FileType.Other
                        title : "Select Picture"
                        directories : ["/accounts/1000/shared/misc"]
                        onFileSelected : {
                            console.log("FileSelected signal received : " + selectedFiles);
                            
                            //make sure to prepend "file://" when using as a source for an ImageView or MediaPlayer
                            //myImageView.imageSource = "file://" + selectedFiles[0];
                        
                        }
                    },
                    SystemDialog {
                       id: installCertsDialog
                       title: "Install certificates for gfycat.com?"
                       body: "gfycat.com certificates are missing on your device and needs to be imported, do you want to import them?"
                       onFinished: {
                           if (result == SystemUiResult.ConfirmButtonSelection) {
                               app.installCertsFinished.connect(onInstallCertsFinished)
                               app.installCerts()
                           }
                           installCertsDialog.cancel();
                       }
                    }
                ]
            }
        }
        
        Container {
            horizontalAlignment: HorizontalAlignment.Center
            verticalAlignment: VerticalAlignment.Top
            ProgressIndicator {
                toValue: 100
                value: 0
                preferredWidth: Infinity
                id: progress
                bottomMargin: 0
                topMargin: 0
            }
        }
        /*attachedObjects: LayoutUpdateHandler {
            onLayoutFrameChanged: {
                //webView.preferredHeight = layoutFrame.height;
                //webView.preferredWidth = layoutFrame.width;
            }
        }*/
    }
    onPostIDChanged: {
        commentsAction.setEnabled(true);
    }
    onWebUrlChanged: {
        var getDesktopModeSetting = Qt.app.getSavedValue("desktopMode", "");
        if (getDesktopModeSetting == "true") {
            webView.settings.setUserAgent(desktopUserAgentString);
        } else {
            if (webUrl.indexOf("imgur.com")!= -1) {
                webView.settings.setUserAgent(desktopUserAgentString);
            } else {
                webView.settings.setUserAgent(mobileUserAgentString);
            }
        }
        console.log(webView.settings.userAgent);
        webView.url = webUrl;
    }
}
