import bb.cascades 1.2
import bb.system 1.2
import Network.SubHttp 1.0
import Network.PostActions 1.0
import Network.FeedHttpCalls 1.0

Page {
    id: searchView
    property string searchTerm
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    titleBar: TitleBar {
        title: "Search"
    }
    attachedObjects: [
        ComponentDefinition {
            id: feedView
            source: "FeedView.qml"
        },
        SubHttp {
            id: subHttp
        }
    ]
    Container {
        Container {
            horizontalAlignment: HorizontalAlignment.Center
            TextField {
                id: textField
                hintText: "Search Reddit"
                input {
                    submitKey: SubmitKey.Search
                    onSubmitted: {
                        if (segControl.selectedIndex == 0) {
                            feedHttp.getSearchResults("", searchIndicator, postSearchList, textField.text);
                        } else {
                            feedHttp.getSearchResults("reddits/", searchIndicator, subSearchList, textField.text);
                        }
                    }
                }
                attachedObjects: [
                    FeedHttpCalls {
                        id: feedHttp
                    }
                ]
            }
            SegmentedControl {
                id: segControl
                options: [
                    Option {
                        text: "posts"
                        description: ""
                    },
                    Option {
                        text: "subreddits"
                        description: "reddits/"
                    }
                ]
                onSelectedIndexChanged: {
                    if (selectedIndex == 0) {
                        postSearchList.visible = true;
                        subSearchList.visible = false;
                        if (textField.text != "") {
                            feedHttp.getSearchResults("", searchIndicator, postSearchList, textField.text);
                        }
                    } else {
                        postSearchList.visible = false;
                        subSearchList.visible = true;
                        if (textField.text != "") {
                            feedHttp.getSearchResults("reddits/", searchIndicator, subSearchList, textField.text);
                        }
                    }
                }
            }
        }
        Container {
            layout: DockLayout {
            }
            ListView {
                scrollRole: ScrollRole.Main
                objectName: "postSearchList"
                id: postSearchList
                listItemComponents: [
                    ListItemComponent {
                        id: listItemRoot
                        FeedListItem {
                            id: feedListItem
                            showSubreddit: true
                            contextActions: [
                                ActionSet {
                                    title: ListItemData.data.title
                                    subtitle: ListItemData.data.author
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("u")
                                            }
                                        ]
                                        id: upvoteAction
                                        title: "Upvote"
                                        imageSource: "asset:///images/upvoteIcon.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                loginToast.body = "You must login to vote!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == true) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == false) {
                                                    searchPostActions.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.score += 2;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    searchPostActions.votePost(selectedItem.data.name, "1");
                                                    selectedItem.data.likes = true;
                                                    selectedItem.data.score += 1;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("d")
                                            }
                                        ]
                                        id: downvoteAction
                                        title: "Downvote"
                                        imageSource: "asset:///images/downvoteIcon.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                loginToast.body = "You must login to vote!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.likes == false) {
                                                    return;
                                                }
                                                if (selectedItem.data.likes == true) {
                                                    searchPostActions.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.score -= 2;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    searchPostActions.votePost(selectedItem.data.name, "-1");
                                                    selectedItem.data.likes = false;
                                                    selectedItem.data.score -= 1;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("c")
                                            }
                                        ]
                                        title: "Comments (" + ListItemData.data.num_comments + ")"
                                        imageSource: "asset:///images/commentsIcon.png"
                                        onTriggered: {
                                            var selectedItem = ListItemData;
                                            var page = feedDetailsViewList.createObject();
                                            page.pageTitle = selectedItem.data.title;
                                            page.postID = (selectedItem.data.name);
                                            feedListItem.ListItem.view.pushPageStack(page);
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("s")
                                            }
                                        ]
                                        id: saveAction
                                        title: "Save"
                                        imageSource: "asset:///images/star.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                loginToast.body = "You must login to save posts!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.saved == false) {
                                                    searchPostActions.savePost(selectedItem.data.name, "save");
                                                    selectedItem.data.saved = true;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                } else {
                                                    searchPostActions.savePost(selectedItem.data.name, "unsave");
                                                    selectedItem.data.saved = false;
                                                    feedListItem.ListItem.view.dataModel.replace(feedListItem.ListItem.indexPath, selectedItem);
                                                }
                                            }
                                        }
                                    }
                                    ActionItem {
                                        shortcuts: [
                                            Shortcut {
                                                key: qsTr("h")
                                            }
                                        ]
                                        id: hideAction
                                        title: "Hide"
                                        imageSource: "asset:///images/hide.png"
                                        onTriggered: {
                                            if (feedListItem.ListItem.view.checkLoggedIn() == false) {
                                                loginToast.body = "You must login to hide posts!";
                                                loginToast.show();
                                            } else {
                                                var selectedItem = ListItemData;
                                                if (selectedItem.data.hidden == false) {
                                                    postActions.hidePost(true, selectedItem.data.name);
                                                    feedListItem.ListItem.view.dataModel.removeAt(feedListItem.ListItem.indexPath);
                                                } else {
                                                    postActions.hidePost(false, selectedItem.data.name);
                                                    feedListItem.ListItem.view.dataModel.removeAt(feedListItem.ListItem.indexPath);
                                                }
                                            }
                                        }
                                    }
                                    InvokeActionItem {
                                        id: shareAction
                                        objectName: "shareAction"
                                        title: "Share"
                                        query {
                                            mimeType: "text/plain"
                                            invokeActionId: "bb.action.SHARE"
                                        }
                                        onTriggered: {
                                            data = ListItemData.data.title + " - https://www.reddit.com" + ListItemData.data.permalink + " via Reddit in Motion";
                                        }
                                    }
                                    attachedObjects: [
                                        SystemToast {
                                            id: loginToast
                                            body: "You must login to vote!"
                                        },
                                        PostActions {
                                            id: searchPostActions
                                        }
                                    ]
                                }
                            ]
                            attachedObjects: [
                                ComponentDefinition {
                                    id: feedDetailsViewList
                                    source: "FeedDetails.qml"
                                }
                            ]
                            contextMenuHandler: ContextMenuHandler {
                                id: contextHandler
                                onPopulating: {
                                    if (ListItemData.data.likes == true) {
                                        upvoteAction.setImageSource("asset:///images/upvoteSelectedIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    } else if (ListItemData.data.likes == false) {
                                        downvoteAction.setImageSource("asset:///images/downvoteSelectedIcon.png");
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                    } else {
                                        upvoteAction.setImageSource("asset:///images/upvoteIcon.png");
                                        downvoteAction.setImageSource("asset:///images/downvoteIcon.png");
                                    }
                                    if (ListItemData.data.saved == true) {
                                        saveAction.setImageSource("asset:///images/saved.png");
                                        saveAction.setTitle("Unsave");
                                    } else {
                                        saveAction.setImageSource("asset:///images/star.png");
                                        saveAction.setTitle("Save");
                                    }
                                    if (ListItemData.data.hidden == true) {
                                        hideAction.setImageSource("asset:///images/unhide.png");
                                        hideAction.setTitle("Unhide");
                                    } else {
                                        hideAction.setImageSource("asset:///images/hide.png");
                                        hideAction.setTitle("Hide");
                                    }
                                }
                            }
                            onCommentBubbleClicked: {
                                var selectedItem = data;
                                var page = feedDetailsViewList.createObject();
                                page.pageTitle = selectedItem.data.title;
                                page.postID = (selectedItem.data.name);
                                feedListItem.ListItem.view.pushPageStack(page);
                            }
                        }
                    }

                ]
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    var domain = selectedItem.data.domain;
                    var found = domain.search('self');
                    if (found != -1) {
                        var page = feedDetailsView.createObject();
                        //page.selfText = selectedItem.data.selftext;
                    } else {
                        var page = Qt.webView.createObject();
                        page.webUrl = selectedItem.data.url;
                        //page.fromSearchView = true;
                    }
                    page.pageTitle = selectedItem.data.title;
                    page.postID = selectedItem.data.name;
                    app.addClicked(selectedItem.data.name);
                    selectedItem.data.clicked = true;
                    dataModel.replace(indexPath, selectedItem);
                    searchView.parent.push(page);
                }
                function pushPageStack(pageParam) {
                    searchView.parent.push(pageParam);
                }
                function checkLoggedIn() {
                    return app.getLoggedIn();
                }
                attachedObjects: [
                    /*ComponentDefinition {
                     * id: webView
                     * source: "PostWebView.qml"
                     },*/
                    ComponentDefinition {
                        id: feedDetailsView
                        source: "FeedDetails.qml"
                    }
                ]
                onDataModelChanged: {
                    if (dataModel.size() <= 0) {
                        emptyLabelMessages.setVisible(true);
                    } else {
                        emptyLabelMessages.setVisible(false);
                    }
                }
            }
            ListView {
                scrollRole: ScrollRole.Main
                visible: false
                objectName: "subSearchList"
                id: subSearchList
                listItemComponents: [
                    ListItemComponent {
                        id: subListItemRoot
                        StandardListItem {
                            id: standardListItem
                            visible: checkHideNSFW()
                            title: ListItemData.data.title
                            description: removeNewLines()
                            status: (ListItemData.data.url).slice(0, -1)
                            function removeNewLines() {
                                var noLines = ListItemData.data.public_description
                                var intIndexOfMatch = noLines.indexOf("\n");
                                while (intIndexOfMatch != -1) {
                                    noLines = noLines.replace("\n", " ");
                                    intIndexOfMatch = noLines.indexOf("\n");
                                }
                                return noLines;
                            }
                            function checkHideNSFW() {
                                //console.log("subreddit nsfw -- " + ListItemData.data);
                                if (ListItemData.data.over18 == true) {
                                    var value = Qt.app.getSavedValue("hideNSFW", "");
                                    if (value == "true") {
                                        return false;
                                    } else {
                                        return true;
                                    }
                                }
                            }
                            contextActions: ActionSet {
                                function removeNewLines() {
                                    var noLines = ListItemData.data.public_description
                                    var intIndexOfMatch = noLines.indexOf("\n");
                                    while (intIndexOfMatch != -1) {
                                        noLines = noLines.replace("\n", " ");
                                        intIndexOfMatch = noLines.indexOf("\n");
                                    }
                                    return noLines;
                                }
                                title: ListItemData.data.title
                                subtitle: removeNewLines()
                                ActionItem {
                                    id: subscribeAction
                                    title: "Unsubscribe"
                                    imageSource: "asset:///images/unsubscribe.png"
                                    onTriggered: {
                                        if (standardListItem.ListItem.view.checkLoggedIn() == false) { //check if logged in, if not, dont allow subscribing
                                            loginToastSubs.show();
                                        } else {
                                            if (subscribeAction.title == "Unsubscribe") {
                                                feedHttpSubs.changeSubscription("unsub", ListItemData.data.name);
                                                standardListItem.ListItem.view.removeItemFromSubscriptionModel(ListItemData.data.name);
                                            } else {
                                                feedHttpSubs.changeSubscription("sub", ListItemData.data.name);
                                                standardListItem.ListItem.view.addItemToSubscriptionModel(standardListItem.ListItem.data);
                                            }
                                        }
                                    }
                                    attachedObjects: [
                                        FeedHttpCalls {
                                            id: feedHttpSubs
                                        },
                                        SystemToast {
                                            id: loginToastSubs
                                            body: "You must login to change subscriptions!"
                                        }
                                    ]
                                }
                                ActionItem {
                                    id: addFavoriteAction
                                    title: "Add to Favorites"
                                    imageSource: "asset:///images/star.png"
                                    onTriggered: {
                                        console.log(ListItemData);
                                        standardListItem.ListItem.view.addFavSub(ListItemData);
                                    }
                                }
                            }
                            contextMenuHandler: ContextMenuHandler {
                                id: subContextHandler
                                onPopulating: {
                                    if (standardListItem.ListItem.view.getSubscribed(ListItemData)) {
                                        subscribeAction.setImageSource("asset:///images/unsubscribe.png");
                                        subscribeAction.setTitle("Unsubscribe");
                                    } else {
                                        subscribeAction.setImageSource("asset:///images/subscribe.png");
                                        subscribeAction.setTitle("Subscribe");
                                    }
                                }
                            }
                        }
                    }
                ]
                function getSubscribed(item) {
                    return subHttp.isSubscribedTo(item);
                }
                function checkLoggedIn() {
                    return Qt.app.getLoggedIn();
                }
                function removeItemFromSubscriptionModel(name) {
                    subHttp.removeItemFromSubscriptionModel(name);
                }
                function addItemToSubscriptionModel(dataModel) {
                    subHttp.addItemToSubscriptionModel(dataModel);
                }
                function addFavSub(item) {
                    Qt.app.addFavoriteSub(item);
                }
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    var page = feedView.createObject();
                    page.subUrl = selectedItem.data.url;
                    searchView.parent.push(page);
                }
                onDataModelChanged: {
                    if (dataModel.size() <= 0) {
                        emptyLabelMessages.setVisible(true);
                    } else {
                        emptyLabelMessages.setVisible(false);
                    }
                }
            }
            ActivityIndicator {
                objectName: "searchIndicator"
                id: searchIndicator
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                preferredWidth: 200
                preferredHeight: 200
            }
            Label {
                id: emptyLabelMessages
                visible: false
                text: "there doesn't seem to be anything here"
                textStyle.color: Color.Red
                multiline: true
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: HorizontalAlignment.Center
                leftMargin: 20
                rightMargin: 20
            }
        }
    }
    onSearchTermChanged: {
        textField.setText(searchTerm);
        feedHttp.getSearchResults("", searchIndicator, postSearchList, textField.text);
    }
}
