import bb.cascades 1.0
import bb.system 1.0
import Network.LoginHttp 1.0
import Network.MessagesHttp 1.0
//import org.domisy 1.0

Page {
    objectName: "settingsSheet"
    id: settingsView
    property alias loginLabel: loginLabel.text
    signal back()
    signal loginClick()
    titleBar: TitleBar {
        title: "Settings"
        dismissAction: ActionItem {
            title: "Back"
            onTriggered: {
                // Cancel has been pressed so we emit the cancel signal (will hide the sheet).
                settingsView.back();
            }
        }
    }
    ScrollView {

        Container {
            //topPadding: 20
            //horizontalAlignment: HorizontalAlignment.Center
            //verticalAlignment: VerticalAlignment.Center
            layout: StackLayout {
            }
            Divider {
                topMargin: 0
                bottomMargin: 0
            }
            Header {
                title: "Login"
            }
            Container {
                topPadding: 20
                bottomPadding: 20
                leftPadding: 15
                rightPadding: 15
                horizontalAlignment: HorizontalAlignment.Center
                ActivityIndicator {
                    id: loginIndicator
                    objectName: "loginIndicator"
                    //running: true
                    visible: false
                    horizontalAlignment: HorizontalAlignment.Center
                }
                Label {
                    id: loginLabel
                    text: "You are not currently logged in."
                    textStyle.textAlign: TextAlign.Center
                    multiline: true
                    function onUsernameRetrieved(pUsername) {
                        loginIndicator.visible = false;
                        text = "You are currently logged in as " + pUsername + ".";
                        textStyle.color = Color.Green;
                    }
                }
                Button {
                    id: loginButton
                    horizontalAlignment: HorizontalAlignment.Center
                    text: "Login"
                    onClicked: {
                        //loginHttp.promptForLogin();
                        app.loginInvoked.connect(loginButton.onLoginInvoked);
                        loginHttp.launchLoginProcess();
                    }
                    function onLoginInvoked(pUrl) {
                        loginHttp.handleRedirect(pUrl);
                    }

                }
                Button {
                    id: logoutButton
                    horizontalAlignment: HorizontalAlignment.Center
                    visible: false
                    text: "Logout"
                    onClicked: {
                        settingsView.logout()
                    }
                }
            }

            Divider {
                topMargin: 0
                bottomMargin: 0
            }
            Header {
                title: "General"
            }
            Container {
                topPadding: 20
                bottomPadding: 20
                leftPadding: 20
                rightPadding: 20
                horizontalAlignment: HorizontalAlignment.Center
                DropDown {
                    property bool firstTime: true
                    title: "Font Size:"
                    selectedIndex: checkFontSetting()
                    options: [
                        Option {
                            text: "Small"
                        },
                        Option {
                            text: "Medium"
                        },
                        Option {
                            text: "Large"
                        }
                    ]
                    onSelectedOptionChanged: {
                        app.saveValue("fontsize", selectedOption.text);
                        if (! firstTime) {
                            changesDialog.show()
                        } else {
                            firstTime = false;
                        }
                    }
                    function checkFontSetting() {
                        var value = app.getSavedValue("fontsize", "");
                        if (value == "Large") {
                            return 2;
                        } else if (value == "Medium") {
                            return 1;
                        } else return 0;
                    }
                    attachedObjects: [
                        SystemToast {
                            id: changesDialog
                            body: "Changes will take effect after refreshing the list."
                        }
                    ]
                }
                Divider {

                }
                Container {
                    layout: DockLayout {
                    }
                    horizontalAlignment: HorizontalAlignment.Fill
                    Label {
                        horizontalAlignment: HorizontalAlignment.Left
                        textStyle.fontSize: FontSize.Large
                        text: "Night Mode"
                    }
                    ToggleButton {
                        property bool isTouched
                        id: nightModeToggle
                        checked: checkNightMode()
                        horizontalAlignment: HorizontalAlignment.Right
                        onCheckedChanged: {
                            if (isTouched) {
                                themeChangeDialog.show();
                                if (checked) {
                                    app.saveValue("theme", "dark");
                                    //app.changeTheme(2);
                                } else {
                                    app.saveValue("theme", "bright");
                                    //app.changeTheme(1);
                                }
                            }
                        }
                        onTouch: {
                            isTouched = true;
                        }
                        function checkNightMode() {
                            var value = app.getSavedValue("theme", "");
                            if (value == "bright") {
                                return false;
                            } else {
                                return true;
                            }
                        }
                    }
                    attachedObjects: [
                        SystemToast {
                            id: themeChangeDialog
                            body: "Changes will take effect after restarting the app."
                        }
                    ]
                }
                Divider {

                }
                CheckBox {
                    checked: checkShowNSFW()
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1.0
                    }

                    text: "Hide NSFW Content"
                    onCheckedChanged: {
                        app.saveValue("hideNSFW", checked.toString());
                    }
                    function checkShowNSFW() {
                        var value = app.getSavedValue("hideNSFW", "");
                        if (value == "true") {
                            return true;
                        } else return false;
                    }
                }
                Divider {

                }
                CheckBox {
                    checked: checkShowSubreddit()
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1.0
                    }

                    text: "Enable Peek in WebView"
                    onCheckedChanged: {
                        app.saveValue("webViewPeekEnabled", checked.toString());
                    }
                    function checkShowSubreddit() {
                        var value = app.getSavedValue("webViewPeekEnabled", "");
                        if (value == "true") {
                            return true;
                        } else return false;
                    }
                }
                Divider {
                
                }
                CheckBox {
                    checked: checkDesktopMode()
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1.0
                    }
                    
                    text: "Desktop Mode in WebView"
                    onCheckedChanged: {
                        app.saveValue("desktopMode", checked.toString());
                    }
                    function checkDesktopMode() {
                        var value = app.getSavedValue("desktopMode", "");
                        if (value == "true") {
                            return true;
                        } else return false;
                    }
                }
                Divider {

                }
                CheckBox {
                    checked: checkOpenLinksBrowser()
                    layoutProperties: StackLayoutProperties {
                        spaceQuota: 1.0
                    }

                    text: "Open Links in Browser"
                    onCheckedChanged: {
                        app.saveValue("openLinksBrowser", checked.toString());
                    }
                    function checkOpenLinksBrowser() {
                        var value = app.getSavedValue("openLinksBrowser", "");
                        if (value == "true") {
                            return true;
                        } else return false;
                    }
                }

            }

            Divider {
                topMargin: 0
                bottomMargin: 0
            }
            Header {
                title: "Cache"
            }
            Container {
                topPadding: 20
                bottomPadding: 20
                horizontalAlignment: HorizontalAlignment.Center
                Container {
                    layout: StackLayout {
                        orientation: LayoutOrientation.LeftToRight
                    }
                    ActivityIndicator {
                        id: cacheActivity
                        visible: false
                        running: true
                    }
                    Label {
                        horizontalAlignment: HorizontalAlignment.Left
                        verticalAlignment: VerticalAlignment.Center
                        id: cacheSizeLabel
                        text: CacheCleaner.imageCacheSize()
                    }
                    Button {
                        id: cacheClearButton
                        horizontalAlignment: HorizontalAlignment.Right
                        verticalAlignment: VerticalAlignment.Center
                        text: "Clear Cache"
                        onClicked: {
                            CacheCleaner.setOnlyCalculate(false);
                            settingsView.threadStarted();
                            settingsView.startThread();
                            cacheClearedToast.show();
                        }
                        attachedObjects: [
                            SystemToast {
                                id: cacheClearedToast
                                body: qsTr("Cache Cleared")
                            }
                        ]
                    }
                }
            }
            Divider {
                topMargin: 0
                bottomMargin: 0
            }
        }
    }
    attachedObjects: [
        LoginHttp {
            id: loginHttp
            onSending: {
                loginButton.enabled = false;
                loginIndicator.start();
                loginIndicator.visible = true;
            }
            onComplete: {
                if (success == 0) {
                    loginButton.visible = false;
                    logoutButton.visible = true;
                    app.usernameRetrieved.connect(loginLabel.onUsernameRetrieved);
                    app.getUsername();
                    Qt.tab0.refreshFeed();
                    messagesHttp.getUnreadCount(Qt.tab3);
                    Qt.tab2.enabled = true;
                    Qt.tab3.enabled = true;
                    Qt.tab2.description = "";
                    Qt.tab3.description = "";
                    if (Qt.subListDropDown) {
                        Qt.subListDropDown.setSelectedIndex(1);
                        Qt.subscribedSubredditOption.setEnabled(true);
                    }
                    app.startTokenRefreshTimer();
                } else if (success == 1) {
                    loginButton.visible = true;
                    logoutButton.visible = false;
                    loginButton.enabled = true;
                    loginLabel.text = "Invalid credentials! Try again."
                    loginIndicator.visible = false;
                } else {
                    loginButton.visible = true;
                    logoutButton.visible = false;
                    loginButton.enabled = true;
                    loginLabel.text = "Error connecting to network! Check your settings and try again."
                    loginIndicator.visible = false;
                }
            }
        },
        MessagesHttp {
            id: messagesHttp
        }
    ]
    function logout() {
        loginHttp.logout();
        loginButton.visible = true;
        logoutButton.visible = false;
        loginButton.enabled = true;
        loginLabel.textStyle.resetColor();
        loginLabel.text = "You are not currently logged in.";
        Qt.tab0.refreshFeed();
        //app.initiateSubRequest(false);
        Qt.tab2.enabled = false;
        Qt.tab3.enabled = false;
        Qt.tab2.description = "Login required";
        Qt.tab3.description = "Login required";
        Qt.tabbedPane.activeTab = Qt.tab0;
        if (Qt.subListDropDown) {
            Qt.subListDropDown.setSelectedIndex(0);
            Qt.subscribedSubredditOption.setEnabled(false);
        }
    }
    function startThread() {
        if (! CacheCleaner.threadRunning()) {
            CacheCleaner.start()
        }
    }
    function threadStarted() {
        cacheClearButton.enabled = false;
        cacheSizeLabel.enabled = false;
        cacheSizeLabel.text = "Computing";
        cacheActivity.visible = true;
    }
    function threadFinished() {
        cacheClearButton.enabled = true;
        cacheActivity.visible = false;
        cacheSizeLabel.text = CacheCleaner.imageCacheSize();
        cacheSizeLabel.enabled = true;
    }
    onCreationCompleted: {
        if (app.getLoggedIn()) {
            var currentUsername = app.getSavedUsername();
            loginButton.visible = false;
            loginLabel.textStyle.color = Color.Green;
            loginLabel.text = "You are currently logged in as " + currentUsername + ".";
            loginIndicator.visible = false;
            logoutButton.visible = true;
        }
        //CacheCleaner.started.connect(settingsView.threadStarted)
        CacheCleaner.finished.connect(settingsView.threadFinished)
        if (CacheCleaner.threadRunning()) {
            threadStarted()
        } else {
            threadStarted();
            startThread() // Get the cache size
        }
        //app.loginExpired.connect(settingsView.logout);
    }
}
