import bb.cascades 1.2
import bb.system 1.2
import Network.SubHttp 1.0
import Network.FeedHttpCalls 1.0

Page {
    id: subredditPage
    actionBarAutoHideBehavior: ActionBarAutoHideBehavior.HideOnScroll
    actionBarVisibility: ChromeVisibility.Overlay
    titleBar: TitleBar {
        id: titlebar1
        title: "my subreddits"
        kind: TitleBarKind.FreeForm
        kindProperties: FreeFormTitleBarKindProperties {
            content: Container {
                leftPadding: 15
                topPadding: 25
                background: Color.create("#0C87BF")
                Label {
                    text: "my subreddits"
                    textStyle.fontSize: FontSize.Large
                    textStyle.color: Color.White
                }
            }
        }
    }
    actions: [
        ActionItem {
            shortcuts: [
                SystemShortcut {
                    type: SystemShortcuts.Search
                }
            ]
            title: "Search"
            imageSource: "asset:///images/searchButton.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                //var selectedItem = dataModel.data(indexPath);
                var page = searchView.createObject();
                navigationPane.push(page);
            }
            attachedObjects: ComponentDefinition {
                id: searchView
                source: "SearchView.qml"
            }
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("g")
                }
            ]
            title: "Goto"
            imageSource: "asset:///images/gotoIcon.png"
            ActionBar.placement: ActionBarPlacement.OnBar
            onTriggered: {
                gotoPrompt.show();
            }
            attachedObjects: [
                SystemPrompt {
                    id: gotoPrompt
                    title: "Goto Subreddit"
                    //inputField.emptyText: "/r/subreddit"
                    body: "ex. /r/subreddit"
                    onFinished: {
                        if (result == SystemUiResult.ConfirmButtonSelection) {
                            var page = feedViewComponent.createObject();
                            var subUrl = inputFieldTextEntry().toLowerCase();
                            var firstSlash = subUrl.lastIndexOf("/");
                            if (firstSlash == -1) {
                                page.subUrl = "/r/" + subUrl + "/";
                            } else {
                                page.subUrl = "/r" + subUrl.substring(firstSlash) + "/";
                            }
                            navigationPane.push(page);
                        }
                    }
                }
            ]
        },
        ActionItem {
            shortcuts: [
                Shortcut {
                    key: qsTr("r")
                }
            ]
            ActionBar.placement: ActionBarPlacement.InOverflow
            title: "Refresh"
            imageSource: "asset:///images/reloadIcon.png"
            onTriggered: {
                if (subDropDown.selectedIndex == 0) {
                    subHttp.initiateSubRequest(false, activity, sublist);
                } else if (subDropDown.selectedIndex == 1) {
                    subHttp.initiateSubRequest(true, activity, sublist);
                } else {
                    Qt.app.getFavoriteSubs(activity, sublist);
                }
            }
        }
    ]
    Container {
        layout: DockLayout {
        }
        Container {
            topPadding: 10
            SegmentedControl {
                id: subDropDown
                property bool firstTimeDropDownChanged: true
                horizontalAlignment: HorizontalAlignment.Center
                verticalAlignment: VerticalAlignment.Top
                selectedIndex: selectSubSection()
                function selectSubSection() {
                    var section = app.getSavedValue("subredditSection", "0");
                    console.log("subsection -- " + section);
                    if (section == "0") {
                        firstTimeDropDownChanged = false;
                        return 0;
                    } else if (section == "1")
                        return 1;
                    else if (section == "2")
                        return 2;
                }
                options: [
                    Option {
                        text: "Default"
                    },
                    Option {
                        id: subscribedSubredditOption
                        text: "Subscribed"
                        enabled: app.getLoggedIn()
                    },
                    Option {
                        text: "Favorites"
                    }
                ]
                onSelectedIndexChanged: {
                    if (firstTimeDropDownChanged) {
                        console.log("firstTimeDropDownChanged");
                        firstTimeDropDownChanged = false;
                    } else {
                        if (selectedIndex == 0) {
                            subHttp.initiateSubRequest(false, activity, sublist);
                            app.saveValue("subredditSection", "0");
                        } else if (selectedIndex == 1) {
                            subHttp.initiateSubRequest(true, activity, sublist);
                            app.saveValue("subredditSection", "1");
                        } else {
                            Qt.app.getFavoriteSubs(activity, sublist);
                            app.saveValue("subredditSection", "2");
                        }
                    }
                }
                onCreationCompleted: {
                    Qt.subListDropDown = subDropDown;
                    Qt.subscribedSubredditOption = subscribedSubredditOption;
                }
                accessibility.name: "Subreddit list selector"
            }
            ListView {
                dataModel: GroupDataModel {
                    id: theModel
                    sortingKeys: [ "group", "main_sort_key" ]
                }
                scrollRole: ScrollRole.Main
                property int indexPathForContext
                objectName: "sublist"
                id: sublist
                listItemComponents: [
                    ListItemComponent {
                        id: listItemComponentID
                        type: "item"
                        StandardListItem {
                            imageSpaceReserved: false
                            id: standardListItem
                            visible: checkHideNSFW()
                            title: ListItemData.data.display_name
                            function checkHideNSFW() {
                                if (ListItemData.data.over18 == true) {
                                    var value = Qt.app.getSavedValue("hideNSFW", "");
                                    if (value == "true") {
                                        return false;
                                    } else {
                                        return true;
                                    }
                                }
                            }
                            contextActions: ActionSet {
                                function removeNewLines() {
                                    var noLines = ListItemData.data.public_description
                                    var intIndexOfMatch = noLines.indexOf("\n");
                                    while (intIndexOfMatch != -1) {
                                        noLines = noLines.replace("\n", " ");
                                        intIndexOfMatch = noLines.indexOf("\n");
                                    }
                                    return noLines;
                                }
                                title: ListItemData.data.title
                                subtitle: removeNewLines()
                                ActionItem {
                                    id: subscribeAction
                                    title: "Unsubscribe"
                                    imageSource: "asset:///images/unsubscribe.png"
                                    onTriggered: {
                                        if (standardListItem.ListItem.view.checkLoggedIn() == false) { //check if logged in, if not, dont allow subscribing
                                            loginToast.show();
                                        } else {
                                            if (Qt.subListDropDown.selectedIndex == 1) {
                                                feedHttp.changeSubscription("unsub", ListItemData.data.name);
                                                standardListItem.ListItem.view.dataModel.removeAt(standardListItem.ListItem.indexPath);
                                                standardListItem.ListItem.view.removeItemFromSubscriptionModel(ListItemData.data.name);
                                            } else {
                                                if (subscribeAction.title == "Unsubscribe") {
                                                    feedHttp.changeSubscription("unsub", ListItemData.data.name);
                                                    standardListItem.ListItem.view.removeItemFromSubscriptionModel(ListItemData.data.name);
                                                } else {
                                                    feedHttp.changeSubscription("sub", ListItemData.data.name);
                                                    standardListItem.ListItem.view.addItemToSubscriptionModel(standardListItem.ListItem.data);
                                                }
                                            }
                                        }
                                    }
                                    attachedObjects: [
                                        FeedHttpCalls {
                                            id: feedHttp
                                        },
                                        SystemToast {
                                            id: loginToast
                                            body: "You must login to change subscriptions!"
                                        }
                                    ]

                                }
                                ActionItem {
                                    id: addFavoriteAction
                                    title: "Add to Favorites"
                                    imageSource: "asset:///images/star.png"
                                    onTriggered: {
                                        //console.log(ListItemData);
                                        if (Qt.subListDropDown.selectedIndex == 2) {
                                            standardListItem.ListItem.view.addFavSub(ListItemData, 1);
                                        } else {
                                            standardListItem.ListItem.view.addFavSub(ListItemData, 0);
                                        }
                                    }
                                }
                            }
                            contextMenuHandler: ContextMenuHandler {
                                id: contextHandler
                                onPopulating: {
                                    if (Qt.subListDropDown.selectedIndex == 1) {
                                        subscribeAction.setImageSource("asset:///images/unsubscribe.png");
                                        subscribeAction.setTitle("Unsubscribe");
                                    } else {
                                        if (standardListItem.ListItem.view.getSubscribed(ListItemData)) {
                                            subscribeAction.setImageSource("asset:///images/unsubscribe.png");
                                            subscribeAction.setTitle("Unsubscribe");
                                        } else {
                                            subscribeAction.setImageSource("asset:///images/subscribe.png");
                                            subscribeAction.setTitle("Subscribe");
                                        }
                                    }
                                    if (Qt.subListDropDown.selectedIndex == 2) {
                                        addFavoriteAction.setTitle("Remove from Favorites");
                                    } else {
                                        addFavoriteAction.setTitle("Add to Favorites");
                                    }
                                }
                            }
                        }
                    }
                ]
                function getSubscribed(item) {
                    return subHttp.isSubscribedTo(item);
                }
                function checkLoggedIn() {
                    return app.getLoggedIn();
                }
                function removeItemFromSubscriptionModel(name) {
                    subHttp.removeItemFromSubscriptionModel(name);
                }
                function addItemToSubscriptionModel(dataModel) {
                    subHttp.addItemToSubscriptionModel(dataModel);
                }
                function addFavSub(item, addRemove) {
                    if (addRemove == 0) { //add
                        Qt.app.addFavoriteSub(item);
                    } else { //remove
                        Qt.app.removeFavoriteSub(item);
                    }
                }
                onTriggered: {
                    var selectedItem = dataModel.data(indexPath);
                    var page = feedViewComponent.createObject();
                    page.subUrl = selectedItem.data.url;
                    if (indexPath == 0) {
                        page.feedListShowSub = true;
                    }
                    navigationPane.push(page);
                }
                attachedObjects: ComponentDefinition {
                    id: feedViewComponent
                    source: "FeedView.qml"

                }
                onDataModelChanged: {
                    if (subDropDown.selectedIndex == 2) {
                        if (! dataModel || dataModel.size() <= 0) {
                            //show message that there arent any favroties
                            sublist.visible = false;
                            noFavContainer.visible = true;
                        }
                    } else {
                        sublist.visible = true;
                        noFavContainer.visible = false;
                    }
                }
            }
        }
        Container {
            id: noFavContainer
            visible: false
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            leftPadding: 20
            rightPadding: 20
            Label {
                id: noFavlabel
                text: "No Favorites"
                textStyle.fontSize: FontSize.Large
                textStyle.textAlign: TextAlign.Center
                horizontalAlignment: HorizontalAlignment.Center
            }
            Label {
                id: addFavInstructLabel
                text: "To add to your favorite subreddits, touch and hold the subreddit that you want to add and tap the star icon."
                multiline: true
                textStyle.fontSize: FontSize.Small
                textStyle.textAlign: TextAlign.Center
                textStyle.color: Color.LightGray
            }
        }

        ActivityIndicator {
            id: activity
            objectName: "activity"
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Center
            preferredWidth: 200
            preferredHeight: 200
        }
    }
    attachedObjects: [
        SubHttp {
            id: subHttp
        }
    ]
    onCreationCompleted: {
        var section = app.getSavedValue("subredditSection", "1");
        if (section == 1) {
            if (app.getLoggedIn()) {
                subHttp.initiateSubRequest(true, activity, sublist);
            } else {
                subHttp.initiateSubRequest(false, activity, sublist);
            }
        } else {
            if (app.getLoggedIn()) {
                subHttp.syncSubscriptions();
            }
            if (section == 0) {
                subHttp.initiateSubRequest(false, activity, sublist);
            } else if (section == 2) {
                Qt.app.getFavoriteSubs(activity, sublist);
            }
        }
    }
}
