import bb.cascades 1.2
import bb.system 1.2
import Network.FeedHttpCalls 1.0
import Network.LoginHttp 1.0
import Network.PostActions 1.0
import Network.MessagesHttp 1.0

TabbedPane {
    property string searchString
    onSearchStringChanged: {
        openSearch(searchString);
    }
    id: tabbedPane
    
    Menu.definition: MenuDefinition {
        settingsAction: SettingsActionItem {
            onTriggered: {
                settingsSheet.open();
            }
        }
        helpAction: HelpActionItem {
            imageSource: "asset:///images/ic_info.png"
            title: qsTr("About")
            onTriggered: {
                //make sure about page doesn't open up in landscape. Maybe dont need this line if we can set property specific to the page which is being opened.
                OrientationSupport.supportedDisplayOrientation = SupportedDisplayOrientation.DisplayPortrait;
                aboutPage.open();

            }
        }
        actions: ActionItem {
            title: "Invite"
            imageSource: "asset:///images/ic_bbm.png"
            onTriggered: {
                app.inviteUserToDownloadViaBBM();
            }
        }
        attachedObjects: [
            Sheet {
                id: settingsSheet
                objectName: "settingsSheet"
                SettingsView {
                    onBack: {
                        settingsSheet.close();
                    }
                }
            },
            Sheet {
                id: aboutPage
                AboutPage {
                    onBack: {
                        OrientationSupport.supportedDisplayOrientation = SupportedDisplayOrientation.All;
                        aboutPage.close();
                    }
                }
            },
            ComponentDefinition {
                id: searchViewGlobal
                source: "SearchView.qml"
            },
            Sheet {
                id: composeThreadSheet
                ComposeThreadSheet {
                    id: composeThreadSheetInner
                    currentSub: ""
                    onBack: {
                        composeThreadSheet.close();
                    }
                    onPostThread: {
                        composeThreadSheet.close();
                    }
                }
            }
        ]
    }
    onCreationCompleted: {
        Qt.tabbedPane = tabbedPane;
        console.log("TabbedPane - onCreationCompleted()")
        // enable layout to adapt to the device rotation
        // don't forget to enable screen rotation in bar-bescriptor.xml (Application->Orientation->Auto-orient)
        OrientationSupport.supportedDisplayOrientation = SupportedDisplayOrientation.All;
        Qt.app = app;
        Application.aboutToQuit.connect(onAboutToQuit);
        app.invokeSearch.connect(openSearch);
        app.invokeShare.connect(openShare);
        app.hrefInvoked.connect(hrefInvoked);
        app.showLoginPrompt();
    }
    attachedObjects: [
        ComponentDefinition {
            id: webView
            source: "PostWebView.qml"
            onCreationCompleted: {
                Qt.webView = webView
            }
        },
        ComponentDefinition {
            id: feedDetailsView
            source: "FeedDetails.qml"
        },
        ComponentDefinition {
            id: feedViewComponentHref
            source: "FeedView.qml"
        }
    ]
    function openSearch(searchString) {
        console.debug("open search QML ---");
        var searchPage = searchViewGlobal.createObject();
        searchPage.searchTerm = searchString;
        if (tabbedPane.activeTab == frontPage) {
            frontNavigationPane.push(searchPage);
        } else if (tabbedPane.activeTab == tab1) {
            navigationPane.push(searchPage);
        } else if (tabbedPane.activeTab == tab2) {
            navigationPaneTab2.push(searchPage);
        } else if (tabbedPane.activeTab == tab3) {
            messagesNavPane.push(searchPage);
        }
    }
    function openShare(shareString) {
        console.debug(shareString);
        composeThreadSheetInner.contentValue = shareString;
        composeThreadSheet.open();
    }
    function hrefInvoked(action, content) {
        if (action == "comments") {
            var page = feedDetailsView.createObject();
            page.pageTitle = "";
            page.postID = "t3_" + content;
            tabbedPane.activeTab.content.peekEnabled = false;
            tabbedPane.activeTab.content.push(page);
        } else if (action == "subreddit") {
            var page = feedViewComponentHref.createObject();
            page.subUrl = content + "/";
            page.feedListShowSub = false;
            tabbedPane.activeTab.content.push(page);
        } else if (action == "frontpage") {

        }
    }
    function onAboutToQuit() {
        app.saveClickedString();
        app.saveFavoriteSubs();
        //app.clearWebviewCache();
    }
    Tab {
        id: frontPage
        title: qsTr("front page")
        imageSource: "asset:///images/frontPageIcon.png"
        NavigationPane {
            id: frontNavigationPane
            FeedView {
                id: feedView
                subUrl: "/"
                feedListShowSub: true
            }
            onPopTransitionEnded: {
                if (page.objectName == "webViewPage") {
                    page.webView.storage.clear();
                    app.removePageParent(page.webView);
                }
                app.removePageParent(page);
            }
            onTopChanged: {
                if (app.getSavedValue("webViewPeekEnabled", "") == "true") {
                    peekEnabled = true;
                } else {
                    if (page.webUrl) {
                        peekEnabled = false;
                    } else {
                        peekEnabled = true;
                    }
                }
            }
        }
        onCreationCompleted: {
            Qt.tab0 = frontPage;
        }
        function refreshFeed() {
            feedView.refresh();
        }
    }
    Tab {
        id: tab1
        title: qsTr("subreddits")
        imageSource: "asset:///images/subredditIcon.png"
        delegateActivationPolicy: TabDelegateActivationPolicy.ActivateWhenSelected
        delegate: Delegate {
            NavigationPane {
                id: navigationPane
                SubredditView {

                }
                onPopTransitionEnded: {
                    if (page.objectName == "webViewPage") {
                        page.webView.storage.clear();
                        app.removePageParent(page.webView);
                    }
                    app.removePageParent(page);
                }
                onTopChanged: {
                    if (app.getSavedValue("webViewPeekEnabled", "") == "true") {
                        peekEnabled = true;
                    } else {
                        if (page.webUrl) {
                            peekEnabled = false;
                        } else {
                            peekEnabled = true;
                        }
                    }
                }
            }
        }
        onCreationCompleted: {
            Qt.tab1 = tab1;
        }
    }
    Tab {
        id: tab2
        enabled: app.getLoggedIn()
        description: (app.getLoggedIn()) ? "" : "Login required"
        title: qsTr("my account")
        imageSource: "asset:///images/user.png"
        delegateActivationPolicy: TabDelegateActivationPolicy.ActivateWhenSelected
        delegate: Delegate {
            NavigationPane {
                id: navigationPaneTab2
                AccountView {
                }
                onPopTransitionEnded: {
                    if (page.objectName == "webViewPage") {
                        page.webView.storage.clear();
                        app.removePageParent(page.webView);
                    }
                    app.removePageParent(page);
                }
                onTopChanged: {
                    if (app.getSavedValue("webViewPeekEnabled", "") == "true") {
                        peekEnabled = true;
                    } else {
                        if (page.webUrl) {
                            peekEnabled = false;
                        } else {
                            peekEnabled = true;
                        }
                    }
                }
            }
        }

        onCreationCompleted: {
            Qt.tab2 = tab2;
        }
    }
    Tab {
        id: tab3
        objectName: "tab3"
        enabled: app.getLoggedIn()
        description: (app.getLoggedIn()) ? "" : "Login required"
        imageSource: "asset:///images/briefcase.png"
        title: qsTr("messages")
        delegateActivationPolicy: TabDelegateActivationPolicy.ActivateWhenSelected
        delegate: Delegate {
            NavigationPane {
                id: messagesNavPane
                MessagesView {
                }
                onPopTransitionEnded: {
                    if (page.objectName == "webViewPage") {
                        page.webView.storage.clear();
                        app.removePageParent(page.webView);
                    }
                    app.removePageParent(page);
                }
            }
        }

        onCreationCompleted: {
            Qt.tab3 = tab3;
            Qt.messagesType = "inbox";
            if (app.getLoggedIn()) {
                messagesHttp.getUnreadCount(tab3);
            }
        }
        onTriggered: {
            tab3.setNewContentAvailable(false);
        }
        attachedObjects: [
            MessagesHttp {
                id: messagesHttp
            }
        ]
    }
}
